# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: laikago_interface.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from pybullet_envs.minitaur.robots import timestamp_pb2 as timestamp__pb2
from pybullet_envs.minitaur.robots import vector_pb2 as vector__pb2



DESCRIPTOR = _descriptor.FileDescriptor(
  name='laikago_interface.proto',
  package='minitaur_fluxworks.control',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x17laikago_interface.proto\x12\x1aminitaur_fluxworks.control\x1a\x0ftimestamp.proto\x1a\x0cvector.proto\"\x82\x01\n\x0cMotorCommand\x12\x10\n\x08motor_id\x18\x01 \x01(\r\x12\x10\n\x08position\x18\x02 \x01(\x02\x12\x15\n\rposition_gain\x18\x03 \x01(\x02\x12\x10\n\x08velocity\x18\x04 \x01(\x02\x12\x15\n\rvelocity_gain\x18\x05 \x01(\x02\x12\x0e\n\x06torque\x18\x06 \x01(\x02\"6\n\x03Led\x12\x0e\n\x06leg_id\x18\x01 \x01(\r\x12\t\n\x01r\x18\x02 \x01(\r\x12\t\n\x01g\x18\x03 \x01(\r\x12\t\n\x01\x62\x18\x04 \x01(\r\"\xf6\x02\n\x0eLaikagoCommand\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12L\n\x0c\x63ontrol_mode\x18\x02 \x01(\x0e\x32\x36.minitaur_fluxworks.control.LaikagoCommand.ControlMode\x12?\n\rmotor_command\x18\x03 \x03(\x0b\x32(.minitaur_fluxworks.control.MotorCommand\x12,\n\x03led\x18\x04 \x03(\x0b\x32\x1f.minitaur_fluxworks.control.Led\"x\n\x0b\x43ontrolMode\x12\x1c\n\x18\x43ONTROL_MODE_UNSPECIFIED\x10\x00\x12\x19\n\x15\x43ONTROL_MODE_POSITION\x10\x01\x12\x17\n\x13\x43ONTROL_MODE_TORQUE\x10\x02\x12\x17\n\x13\x43ONTROL_MODE_HYBRID\x10\x03\"\x15\n\x13LaikagoStateRequest\"\xd8\x01\n\x03Imu\x12/\n\nquaternion\x18\x01 \x01(\x0b\x32\x1b.robotics.messages.Vector4f\x12.\n\tgyroscope\x18\x02 \x01(\x0b\x32\x1b.robotics.messages.Vector3f\x12\x31\n\x0c\x61\x63\x63\x65leration\x18\x03 \x01(\x0b\x32\x1b.robotics.messages.Vector3f\x12(\n\x03rpy\x18\x04 \x01(\x0b\x32\x1b.robotics.messages.Vector3f\x12\x13\n\x0btemperature\x18\x05 \x01(\x02\"\xa3\x01\n\nMotorState\x12\x10\n\x08motor_id\x18\x01 \x01(\r\x12\x0c\n\x04mode\x18\x02 \x01(\r\x12\x10\n\x08position\x18\x03 \x01(\x02\x12\x15\n\rposition_gain\x18\x04 \x01(\x02\x12\x10\n\x08velocity\x18\x05 \x01(\x02\x12\x15\n\rvelocity_gain\x18\x06 \x01(\x02\x12\x0e\n\x06torque\x18\x07 \x01(\x02\x12\x13\n\x0btemperature\x18\x08 \x01(\x02\"X\n\x0c\x43ontactState\x12\x0e\n\x06leg_id\x18\x01 \x01(\r\x12\r\n\x05\x66orce\x18\x02 \x01(\x02\x12)\n\x04\x61xis\x18\x03 \x01(\x0b\x32\x1b.robotics.messages.Vector3f\"\xcb\x02\n\x0cLaikagoState\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rcontrol_level\x18\x02 \x01(\r\x12,\n\x03imu\x18\x03 \x01(\x0b\x32\x1f.minitaur_fluxworks.control.Imu\x12;\n\x0bmotor_state\x18\x04 \x03(\x0b\x32&.minitaur_fluxworks.control.MotorState\x12?\n\rcontact_state\x18\x05 \x03(\x0b\x32(.minitaur_fluxworks.control.ContactState\x12#\n\x1bmicrocontroller_time_millis\x18\x06 \x01(\r\x12\x17\n\x0fwireless_remote\x18\x07 \x01(\x0c\x12\x0b\n\x03\x63rc\x18\x08 \x01(\r\"\x8b\x01\n\x13LaikagoCommandState\x12;\n\x07\x63ommand\x18\x01 \x01(\x0b\x32*.minitaur_fluxworks.control.LaikagoCommand\x12\x37\n\x05state\x18\x02 \x01(\x0b\x32(.minitaur_fluxworks.control.LaikagoState\"\x84\x02\n\x17LaikagoHighLevelCommand\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rcontrol_level\x18\x02 \x01(\r\x12\x14\n\x0c\x63ontrol_mode\x18\x03 \x01(\r\x12/\n\nwalk_speed\x18\x04 \x01(\x0b\x32\x1b.robotics.messages.Vector3f\x12\x13\n\x0b\x62ody_height\x18\x05 \x01(\x02\x12\x1d\n\x15\x66oot_clearance_height\x18\x06 \x01(\x02\x12(\n\x03rpy\x18\x07 \x01(\x0b\x32\x1b.robotics.messages.Vector3f\"\xb3\x04\n\x15LaikagoHighLevelState\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rcontrol_level\x18\x02 \x01(\r\x12\x14\n\x0c\x63ontrol_mode\x18\x03 \x01(\r\x12,\n\x03imu\x18\x04 \x01(\x0b\x32\x1f.minitaur_fluxworks.control.Imu\x12/\n\nwalk_speed\x18\x05 \x01(\x0b\x32\x1b.robotics.messages.Vector3f\x12\x13\n\x0b\x62ody_height\x18\x08 \x01(\x02\x12\x15\n\rup_down_speed\x18\t \x01(\x02\x12\x31\n\x0c\x63om_position\x18\n \x01(\x0b\x32\x1b.robotics.messages.Vector3f\x12\x39\n\x14\x66oot_position_to_com\x18\x0b \x03(\x0b\x32\x1b.robotics.messages.Vector3f\x12\x39\n\x14\x66oot_velocity_to_com\x18\x0c \x03(\x0b\x32\x1b.robotics.messages.Vector3f\x12?\n\rcontact_state\x18\r \x03(\x0b\x32(.minitaur_fluxworks.control.ContactState\x12#\n\x1bmicrocontroller_time_millis\x18\x0e \x01(\r\x12\x17\n\x0fwireless_remote\x18\x0f \x01(\x0c\x12\x0b\n\x03\x63rc\x18\x10 \x01(\r\"\x1e\n\x1cLaikagoHighLevelStateRequest\"\xa6\x01\n\x1cLaikagoHighLevelCommandState\x12\x44\n\x07\x63ommand\x18\x01 \x01(\x0b\x32\x33.minitaur_fluxworks.control.LaikagoHighLevelCommand\x12@\n\x05state\x18\x02 \x01(\x0b\x32\x31.minitaur_fluxworks.control.LaikagoHighLevelState2\xed\x01\n\x1bLaikagoControlGrpcInterface\x12\x65\n\x0bSendCommand\x12*.minitaur_fluxworks.control.LaikagoCommand\x1a(.minitaur_fluxworks.control.LaikagoState\"\x00\x12g\n\x08GetState\x12/.minitaur_fluxworks.control.LaikagoStateRequest\x1a(.minitaur_fluxworks.control.LaikagoState\"\x00\x32\x9a\x02\n$LaikagoHighLevelControlGrpcInterface\x12w\n\x0bSendCommand\x12\x33.minitaur_fluxworks.control.LaikagoHighLevelCommand\x1a\x31.minitaur_fluxworks.control.LaikagoHighLevelState\"\x00\x12y\n\x08GetState\x12\x38.minitaur_fluxworks.control.LaikagoHighLevelStateRequest\x1a\x31.minitaur_fluxworks.control.LaikagoHighLevelState\"\x00\x62\x06proto3'
  ,
  dependencies=[timestamp__pb2.DESCRIPTOR,vector__pb2.DESCRIPTOR,])



_LAIKAGOCOMMAND_CONTROLMODE = _descriptor.EnumDescriptor(
  name='ControlMode',
  full_name='minitaur_fluxworks.control.LaikagoCommand.ControlMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CONTROL_MODE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONTROL_MODE_POSITION', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONTROL_MODE_TORQUE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONTROL_MODE_HYBRID', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=530,
  serialized_end=650,
)
_sym_db.RegisterEnumDescriptor(_LAIKAGOCOMMAND_CONTROLMODE)


_MOTORCOMMAND = _descriptor.Descriptor(
  name='MotorCommand',
  full_name='minitaur_fluxworks.control.MotorCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='motor_id', full_name='minitaur_fluxworks.control.MotorCommand.motor_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position', full_name='minitaur_fluxworks.control.MotorCommand.position', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position_gain', full_name='minitaur_fluxworks.control.MotorCommand.position_gain', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity', full_name='minitaur_fluxworks.control.MotorCommand.velocity', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_gain', full_name='minitaur_fluxworks.control.MotorCommand.velocity_gain', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='torque', full_name='minitaur_fluxworks.control.MotorCommand.torque', index=5,
      number=6, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=87,
  serialized_end=217,
)


_LED = _descriptor.Descriptor(
  name='Led',
  full_name='minitaur_fluxworks.control.Led',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='leg_id', full_name='minitaur_fluxworks.control.Led.leg_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='r', full_name='minitaur_fluxworks.control.Led.r', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='g', full_name='minitaur_fluxworks.control.Led.g', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='b', full_name='minitaur_fluxworks.control.Led.b', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=219,
  serialized_end=273,
)


_LAIKAGOCOMMAND = _descriptor.Descriptor(
  name='LaikagoCommand',
  full_name='minitaur_fluxworks.control.LaikagoCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='minitaur_fluxworks.control.LaikagoCommand.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='control_mode', full_name='minitaur_fluxworks.control.LaikagoCommand.control_mode', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='motor_command', full_name='minitaur_fluxworks.control.LaikagoCommand.motor_command', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='led', full_name='minitaur_fluxworks.control.LaikagoCommand.led', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LAIKAGOCOMMAND_CONTROLMODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=276,
  serialized_end=650,
)


_LAIKAGOSTATEREQUEST = _descriptor.Descriptor(
  name='LaikagoStateRequest',
  full_name='minitaur_fluxworks.control.LaikagoStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=652,
  serialized_end=673,
)


_IMU = _descriptor.Descriptor(
  name='Imu',
  full_name='minitaur_fluxworks.control.Imu',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='quaternion', full_name='minitaur_fluxworks.control.Imu.quaternion', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gyroscope', full_name='minitaur_fluxworks.control.Imu.gyroscope', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='acceleration', full_name='minitaur_fluxworks.control.Imu.acceleration', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rpy', full_name='minitaur_fluxworks.control.Imu.rpy', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='temperature', full_name='minitaur_fluxworks.control.Imu.temperature', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=676,
  serialized_end=892,
)


_MOTORSTATE = _descriptor.Descriptor(
  name='MotorState',
  full_name='minitaur_fluxworks.control.MotorState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='motor_id', full_name='minitaur_fluxworks.control.MotorState.motor_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mode', full_name='minitaur_fluxworks.control.MotorState.mode', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position', full_name='minitaur_fluxworks.control.MotorState.position', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position_gain', full_name='minitaur_fluxworks.control.MotorState.position_gain', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity', full_name='minitaur_fluxworks.control.MotorState.velocity', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_gain', full_name='minitaur_fluxworks.control.MotorState.velocity_gain', index=5,
      number=6, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='torque', full_name='minitaur_fluxworks.control.MotorState.torque', index=6,
      number=7, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='temperature', full_name='minitaur_fluxworks.control.MotorState.temperature', index=7,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=895,
  serialized_end=1058,
)


_CONTACTSTATE = _descriptor.Descriptor(
  name='ContactState',
  full_name='minitaur_fluxworks.control.ContactState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='leg_id', full_name='minitaur_fluxworks.control.ContactState.leg_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='force', full_name='minitaur_fluxworks.control.ContactState.force', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='axis', full_name='minitaur_fluxworks.control.ContactState.axis', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1060,
  serialized_end=1148,
)


_LAIKAGOSTATE = _descriptor.Descriptor(
  name='LaikagoState',
  full_name='minitaur_fluxworks.control.LaikagoState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='minitaur_fluxworks.control.LaikagoState.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='control_level', full_name='minitaur_fluxworks.control.LaikagoState.control_level', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='imu', full_name='minitaur_fluxworks.control.LaikagoState.imu', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='motor_state', full_name='minitaur_fluxworks.control.LaikagoState.motor_state', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contact_state', full_name='minitaur_fluxworks.control.LaikagoState.contact_state', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='microcontroller_time_millis', full_name='minitaur_fluxworks.control.LaikagoState.microcontroller_time_millis', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='wireless_remote', full_name='minitaur_fluxworks.control.LaikagoState.wireless_remote', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='crc', full_name='minitaur_fluxworks.control.LaikagoState.crc', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1151,
  serialized_end=1482,
)


_LAIKAGOCOMMANDSTATE = _descriptor.Descriptor(
  name='LaikagoCommandState',
  full_name='minitaur_fluxworks.control.LaikagoCommandState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='command', full_name='minitaur_fluxworks.control.LaikagoCommandState.command', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='state', full_name='minitaur_fluxworks.control.LaikagoCommandState.state', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1485,
  serialized_end=1624,
)


_LAIKAGOHIGHLEVELCOMMAND = _descriptor.Descriptor(
  name='LaikagoHighLevelCommand',
  full_name='minitaur_fluxworks.control.LaikagoHighLevelCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='minitaur_fluxworks.control.LaikagoHighLevelCommand.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='control_level', full_name='minitaur_fluxworks.control.LaikagoHighLevelCommand.control_level', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='control_mode', full_name='minitaur_fluxworks.control.LaikagoHighLevelCommand.control_mode', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='walk_speed', full_name='minitaur_fluxworks.control.LaikagoHighLevelCommand.walk_speed', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body_height', full_name='minitaur_fluxworks.control.LaikagoHighLevelCommand.body_height', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='foot_clearance_height', full_name='minitaur_fluxworks.control.LaikagoHighLevelCommand.foot_clearance_height', index=5,
      number=6, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rpy', full_name='minitaur_fluxworks.control.LaikagoHighLevelCommand.rpy', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1627,
  serialized_end=1887,
)


_LAIKAGOHIGHLEVELSTATE = _descriptor.Descriptor(
  name='LaikagoHighLevelState',
  full_name='minitaur_fluxworks.control.LaikagoHighLevelState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='control_level', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.control_level', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='control_mode', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.control_mode', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='imu', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.imu', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='walk_speed', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.walk_speed', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body_height', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.body_height', index=5,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='up_down_speed', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.up_down_speed', index=6,
      number=9, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='com_position', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.com_position', index=7,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='foot_position_to_com', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.foot_position_to_com', index=8,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='foot_velocity_to_com', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.foot_velocity_to_com', index=9,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contact_state', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.contact_state', index=10,
      number=13, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='microcontroller_time_millis', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.microcontroller_time_millis', index=11,
      number=14, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='wireless_remote', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.wireless_remote', index=12,
      number=15, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='crc', full_name='minitaur_fluxworks.control.LaikagoHighLevelState.crc', index=13,
      number=16, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1890,
  serialized_end=2453,
)


_LAIKAGOHIGHLEVELSTATEREQUEST = _descriptor.Descriptor(
  name='LaikagoHighLevelStateRequest',
  full_name='minitaur_fluxworks.control.LaikagoHighLevelStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2455,
  serialized_end=2485,
)


_LAIKAGOHIGHLEVELCOMMANDSTATE = _descriptor.Descriptor(
  name='LaikagoHighLevelCommandState',
  full_name='minitaur_fluxworks.control.LaikagoHighLevelCommandState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='command', full_name='minitaur_fluxworks.control.LaikagoHighLevelCommandState.command', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='state', full_name='minitaur_fluxworks.control.LaikagoHighLevelCommandState.state', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2488,
  serialized_end=2654,
)

_LAIKAGOCOMMAND.fields_by_name['timestamp'].message_type = timestamp__pb2._TIMESTAMP
_LAIKAGOCOMMAND.fields_by_name['control_mode'].enum_type = _LAIKAGOCOMMAND_CONTROLMODE
_LAIKAGOCOMMAND.fields_by_name['motor_command'].message_type = _MOTORCOMMAND
_LAIKAGOCOMMAND.fields_by_name['led'].message_type = _LED
_LAIKAGOCOMMAND_CONTROLMODE.containing_type = _LAIKAGOCOMMAND
_IMU.fields_by_name['quaternion'].message_type = vector__pb2._VECTOR4F
_IMU.fields_by_name['gyroscope'].message_type = vector__pb2._VECTOR3F
_IMU.fields_by_name['acceleration'].message_type = vector__pb2._VECTOR3F
_IMU.fields_by_name['rpy'].message_type = vector__pb2._VECTOR3F
_CONTACTSTATE.fields_by_name['axis'].message_type = vector__pb2._VECTOR3F
_LAIKAGOSTATE.fields_by_name['timestamp'].message_type = timestamp__pb2._TIMESTAMP
_LAIKAGOSTATE.fields_by_name['imu'].message_type = _IMU
_LAIKAGOSTATE.fields_by_name['motor_state'].message_type = _MOTORSTATE
_LAIKAGOSTATE.fields_by_name['contact_state'].message_type = _CONTACTSTATE
_LAIKAGOCOMMANDSTATE.fields_by_name['command'].message_type = _LAIKAGOCOMMAND
_LAIKAGOCOMMANDSTATE.fields_by_name['state'].message_type = _LAIKAGOSTATE
_LAIKAGOHIGHLEVELCOMMAND.fields_by_name['timestamp'].message_type = timestamp__pb2._TIMESTAMP
_LAIKAGOHIGHLEVELCOMMAND.fields_by_name['walk_speed'].message_type = vector__pb2._VECTOR3F
_LAIKAGOHIGHLEVELCOMMAND.fields_by_name['rpy'].message_type = vector__pb2._VECTOR3F
_LAIKAGOHIGHLEVELSTATE.fields_by_name['timestamp'].message_type = timestamp__pb2._TIMESTAMP
_LAIKAGOHIGHLEVELSTATE.fields_by_name['imu'].message_type = _IMU
_LAIKAGOHIGHLEVELSTATE.fields_by_name['walk_speed'].message_type = vector__pb2._VECTOR3F
_LAIKAGOHIGHLEVELSTATE.fields_by_name['com_position'].message_type = vector__pb2._VECTOR3F
_LAIKAGOHIGHLEVELSTATE.fields_by_name['foot_position_to_com'].message_type = vector__pb2._VECTOR3F
_LAIKAGOHIGHLEVELSTATE.fields_by_name['foot_velocity_to_com'].message_type = vector__pb2._VECTOR3F
_LAIKAGOHIGHLEVELSTATE.fields_by_name['contact_state'].message_type = _CONTACTSTATE
_LAIKAGOHIGHLEVELCOMMANDSTATE.fields_by_name['command'].message_type = _LAIKAGOHIGHLEVELCOMMAND
_LAIKAGOHIGHLEVELCOMMANDSTATE.fields_by_name['state'].message_type = _LAIKAGOHIGHLEVELSTATE
DESCRIPTOR.message_types_by_name['MotorCommand'] = _MOTORCOMMAND
DESCRIPTOR.message_types_by_name['Led'] = _LED
DESCRIPTOR.message_types_by_name['LaikagoCommand'] = _LAIKAGOCOMMAND
DESCRIPTOR.message_types_by_name['LaikagoStateRequest'] = _LAIKAGOSTATEREQUEST
DESCRIPTOR.message_types_by_name['Imu'] = _IMU
DESCRIPTOR.message_types_by_name['MotorState'] = _MOTORSTATE
DESCRIPTOR.message_types_by_name['ContactState'] = _CONTACTSTATE
DESCRIPTOR.message_types_by_name['LaikagoState'] = _LAIKAGOSTATE
DESCRIPTOR.message_types_by_name['LaikagoCommandState'] = _LAIKAGOCOMMANDSTATE
DESCRIPTOR.message_types_by_name['LaikagoHighLevelCommand'] = _LAIKAGOHIGHLEVELCOMMAND
DESCRIPTOR.message_types_by_name['LaikagoHighLevelState'] = _LAIKAGOHIGHLEVELSTATE
DESCRIPTOR.message_types_by_name['LaikagoHighLevelStateRequest'] = _LAIKAGOHIGHLEVELSTATEREQUEST
DESCRIPTOR.message_types_by_name['LaikagoHighLevelCommandState'] = _LAIKAGOHIGHLEVELCOMMANDSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MotorCommand = _reflection.GeneratedProtocolMessageType('MotorCommand', (_message.Message,), {
  'DESCRIPTOR' : _MOTORCOMMAND,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.MotorCommand)
  })
_sym_db.RegisterMessage(MotorCommand)

Led = _reflection.GeneratedProtocolMessageType('Led', (_message.Message,), {
  'DESCRIPTOR' : _LED,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.Led)
  })
_sym_db.RegisterMessage(Led)

LaikagoCommand = _reflection.GeneratedProtocolMessageType('LaikagoCommand', (_message.Message,), {
  'DESCRIPTOR' : _LAIKAGOCOMMAND,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.LaikagoCommand)
  })
_sym_db.RegisterMessage(LaikagoCommand)

LaikagoStateRequest = _reflection.GeneratedProtocolMessageType('LaikagoStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _LAIKAGOSTATEREQUEST,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.LaikagoStateRequest)
  })
_sym_db.RegisterMessage(LaikagoStateRequest)

Imu = _reflection.GeneratedProtocolMessageType('Imu', (_message.Message,), {
  'DESCRIPTOR' : _IMU,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.Imu)
  })
_sym_db.RegisterMessage(Imu)

MotorState = _reflection.GeneratedProtocolMessageType('MotorState', (_message.Message,), {
  'DESCRIPTOR' : _MOTORSTATE,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.MotorState)
  })
_sym_db.RegisterMessage(MotorState)

ContactState = _reflection.GeneratedProtocolMessageType('ContactState', (_message.Message,), {
  'DESCRIPTOR' : _CONTACTSTATE,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.ContactState)
  })
_sym_db.RegisterMessage(ContactState)

LaikagoState = _reflection.GeneratedProtocolMessageType('LaikagoState', (_message.Message,), {
  'DESCRIPTOR' : _LAIKAGOSTATE,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.LaikagoState)
  })
_sym_db.RegisterMessage(LaikagoState)

LaikagoCommandState = _reflection.GeneratedProtocolMessageType('LaikagoCommandState', (_message.Message,), {
  'DESCRIPTOR' : _LAIKAGOCOMMANDSTATE,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.LaikagoCommandState)
  })
_sym_db.RegisterMessage(LaikagoCommandState)

LaikagoHighLevelCommand = _reflection.GeneratedProtocolMessageType('LaikagoHighLevelCommand', (_message.Message,), {
  'DESCRIPTOR' : _LAIKAGOHIGHLEVELCOMMAND,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.LaikagoHighLevelCommand)
  })
_sym_db.RegisterMessage(LaikagoHighLevelCommand)

LaikagoHighLevelState = _reflection.GeneratedProtocolMessageType('LaikagoHighLevelState', (_message.Message,), {
  'DESCRIPTOR' : _LAIKAGOHIGHLEVELSTATE,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.LaikagoHighLevelState)
  })
_sym_db.RegisterMessage(LaikagoHighLevelState)

LaikagoHighLevelStateRequest = _reflection.GeneratedProtocolMessageType('LaikagoHighLevelStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _LAIKAGOHIGHLEVELSTATEREQUEST,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.LaikagoHighLevelStateRequest)
  })
_sym_db.RegisterMessage(LaikagoHighLevelStateRequest)

LaikagoHighLevelCommandState = _reflection.GeneratedProtocolMessageType('LaikagoHighLevelCommandState', (_message.Message,), {
  'DESCRIPTOR' : _LAIKAGOHIGHLEVELCOMMANDSTATE,
  '__module__' : 'laikago_interface_pb2'
  # @@protoc_insertion_point(class_scope:minitaur_fluxworks.control.LaikagoHighLevelCommandState)
  })
_sym_db.RegisterMessage(LaikagoHighLevelCommandState)



_LAIKAGOCONTROLGRPCINTERFACE = _descriptor.ServiceDescriptor(
  name='LaikagoControlGrpcInterface',
  full_name='minitaur_fluxworks.control.LaikagoControlGrpcInterface',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=2657,
  serialized_end=2894,
  methods=[
  _descriptor.MethodDescriptor(
    name='SendCommand',
    full_name='minitaur_fluxworks.control.LaikagoControlGrpcInterface.SendCommand',
    index=0,
    containing_service=None,
    input_type=_LAIKAGOCOMMAND,
    output_type=_LAIKAGOSTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetState',
    full_name='minitaur_fluxworks.control.LaikagoControlGrpcInterface.GetState',
    index=1,
    containing_service=None,
    input_type=_LAIKAGOSTATEREQUEST,
    output_type=_LAIKAGOSTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_LAIKAGOCONTROLGRPCINTERFACE)

DESCRIPTOR.services_by_name['LaikagoControlGrpcInterface'] = _LAIKAGOCONTROLGRPCINTERFACE


_LAIKAGOHIGHLEVELCONTROLGRPCINTERFACE = _descriptor.ServiceDescriptor(
  name='LaikagoHighLevelControlGrpcInterface',
  full_name='minitaur_fluxworks.control.LaikagoHighLevelControlGrpcInterface',
  file=DESCRIPTOR,
  index=1,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=2897,
  serialized_end=3179,
  methods=[
  _descriptor.MethodDescriptor(
    name='SendCommand',
    full_name='minitaur_fluxworks.control.LaikagoHighLevelControlGrpcInterface.SendCommand',
    index=0,
    containing_service=None,
    input_type=_LAIKAGOHIGHLEVELCOMMAND,
    output_type=_LAIKAGOHIGHLEVELSTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetState',
    full_name='minitaur_fluxworks.control.LaikagoHighLevelControlGrpcInterface.GetState',
    index=1,
    containing_service=None,
    input_type=_LAIKAGOHIGHLEVELSTATEREQUEST,
    output_type=_LAIKAGOHIGHLEVELSTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_LAIKAGOHIGHLEVELCONTROLGRPCINTERFACE)

DESCRIPTOR.services_by_name['LaikagoHighLevelControlGrpcInterface'] = _LAIKAGOHIGHLEVELCONTROLGRPCINTERFACE

# @@protoc_insertion_point(module_scope)



"""An env that uses MPC-based motion controller to realize higher level footstep planning."""

import copy
import enum
from typing import Sequence

import dataclasses
import gin
import gym
import numpy as np

from pybullet_envs.minitaur.agents.baseline_controller import com_height_estimator
from pybullet_envs.minitaur.agents.baseline_controller import gait_generator as gait_generator_lib
from pybullet_envs.minitaur.agents.baseline_controller import imu_based_com_velocity_estimator
from pybullet_envs.minitaur.agents.baseline_controller import multi_state_estimator
from pybullet_envs.minitaur.agents.baseline_controller import openloop_gait_generator
from pybullet_envs.minitaur.agents.baseline_controller import torque_stance_leg_controller
from pybullet_envs.minitaur.robots import laikago_kinematic_constants

_UNIT_QUATERNION = (0, 0, 0, 1)
_NUM_LEGS = laikago_kinematic_constants.NUM_LEGS
_MOTORS_PER_LEG = 3
_DEFAULT_BODY_HEIGHT = 0.45
_DEFAULT_BASE_SPEED = (0.0, 0.0)
_DEFAULT_BASE_TWIST_SPEED = 0.0
_DEFAULT_ROLL_PITCH = (0.0, 0.0)
_DEFAULT_SWING_TARGET = (0.0, 0.0, 0.0)
_DEFAULT_SWING_CLERANCE = 0.05
_MOTOR_KP = [220.0] * 12
_MOTOR_KD = [0.3, 2.0, 2.0] * 4
_BASE_VELOCITY_ACTION_RANGE = ((-1.0, -0.1), (1.0, 0.1))
_BASE_TWIST_SPEED_ACTION_RANGE = (-0.5, 0.5)
_BASE_HEIGHT_ACTION_RANGE = (0.3, 0.5)
# Action bound for the swing target in local x, y, z direction.
_SWING_TARGET_ACTION_RANGE = ((-0.3, -0.1, -0.25), (0.3, 0.1, 0.25))
_PITCH_ROLL_ACTION_RANGE = (-0.35, 0.35)
_SWING_CLEARANCE_ACTION_RANGE = (0.05, 0.3)
_SWING_TARGET_DELTA_ACTION_RANGE = ((-0.02, -0.01, -0.02), (0.02, 0.01, 0.02))
_SWING_CLEARANCE_DELTA_RANGE = (-0.02, 0.02)


@gin.configurable
@dataclasses.dataclass
class BaseTargetHorizontalComVelocityHeuristic(object):
  """A class for mapping swing foot targets to a heuristic target com velocity."""
  horizontal_com_velocity_heuristic: np.ndarray = np.zeros(2)

  def update_horizontal_com_velocity_heuristic(self, hip_relative_swing_targets,
                                               com_velocity, swing_durations):
    del hip_relative_swing_targets, com_velocity, swing_durations
    pass

  def reset(self):
    self.horizontal_com_velocity_heuristic = np.zeros(2)


# TODO(magicmelon): Add a one-pager to explain the inverse raibert heuristics.
@gin.configurable
class InverseRaibertTargetHorizontalComVelocityHeuristic(
    BaseTargetHorizontalComVelocityHeuristic):
  """A class for mapping swing foot targets to a target com velocity with Raibert Heuristics."""

  def __init__(self, gains=(-0.25, -0.1)):
    self._gains = np.array(gains)

  def update_horizontal_com_velocity_heuristic(self, hip_relative_swing_targets,
                                               com_velocity, swing_durations):
    assert len(hip_relative_swing_targets) == len(swing_durations)
    target_com_horizontal_velocities = []
    for i in range(len(hip_relative_swing_targets)):
      target_com_horizontal_velocity = (
          com_velocity / 2.0 * swing_durations[i] -
          hip_relative_swing_targets[i]) / self._gains + com_velocity
      target_com_horizontal_velocities.append(target_com_horizontal_velocity)
    if target_com_horizontal_velocities:
      self.horizontal_com_velocity_heuristic = np.mean(
          target_com_horizontal_velocities, axis=0)


@gin.constants_from_enum
class Gait(enum.Enum):
  """The possible gaits."""
  WALK = 0
  TROT = 1


def _select_gait(gait_type=Gait.WALK):
  """Selects a gait pattern.

  Args:
    gait_type: which gait to use.

  Returns:
    A tuple of (stance_duration, duty_factor, initial_phase)
  """
  # Each gait is composed of stance_duration, duty_factor, and
  # init_phase_full_cycle.
  if gait_type == Gait.TROT:
    return [0.3] * 4, [0.6] * 4, [0, 0.5, 0.5, 0]
  elif gait_type == Gait.WALK:
    return [0.75] * 4, [0.8] * 4, [0.25, 0.75, 0.5, 0]
  else:
    raise NotImplementedError


@gin.configurable
class MPCLocomotionWrapper(object):
  """An env that uses MPC-based motion controller to realize footstep planning.

    The env takes as input the target position of the swing feet and the target
    base movements, and internally uses an MPC-based controller to achieve these
    targets. It assumes that the robot follows a given gait pattern, specified
    during initialization.
    Before each foot starts to swing, the env will request from the policy a
    target swing location and height defined in the local frame w.r.t the
    default toe position. During the swing of the foot, the policy can adjust
    the base velocity, height, roll, pitch, and twist. Optionally, the policy
    can also output a delta to the last target swing location to adjust the
    swing trajectory during the swing
    Observations (introduced in this env):
      gait_phases (4D): Normalized phase within the gait cycle for each foot.
      feet_states (4D): State of each foot. -1: stance, 1: swing, -2: lose
        contact, 2: early contact.
      need_new_swing_target (4D): Whether the foot needs a new swing target at
        the current step. Set to 1 when the foot switches to swing from a
        different state. When equals to 0, the corresponding foot target will
        not have effect.
      estimated_base_speed (3D): Estimated base velocity.
      estimated_body_height (1D): Estimated base height.
      heuristics_com_velocity (3D): Target base velocity calculated from the
        input step-length using inverse Raibert heuristics. Used when
        compute_heuristic_com_speed is True.
      current_toe_target: Immediate tracking targets for the four feet in the
        local frame.
    Action components:
      swing_targets (12D): Used in HL_LL and HL_only mode. Specifies
        the swing target for each foot w.r.t the default local toe position.
      swing_clearance (4D): Used in HL_LL and HL_only mode. Specifies
        the height of the highest point in the swing trajectory.
      swing_targets_delta (12D): Used if policy_output_swing_action_delta.
        Specifies the change in the swing target for each foot.
      swing_clearance_delta (4D): Used if policy_output_swing_action_delta.
        Specifies the change in the swing clearance for each foot.
      target_base_velocity (2D): Target base velocity in the horizontal plane.
        Used when compute_heuristic_com_speed is False.
      base_twist (1D): Target base twist.
      base_height (1D): Target base height.
      base_roll_pitch (2D): Target base roll and pitch.

  Attributes:
    observation: The current observation of the environment.
    last_action: The last that was used to step the environment.
    env_step_counter: The number of control steps that have been elapesed since
      the environment is reset.
    action_space: The action space of the environment.
  """

  def __init__(
      self,
      gym_env,
      swing_target_action_range=_SWING_TARGET_ACTION_RANGE,
      swing_clearance_action_range=_SWING_CLEARANCE_ACTION_RANGE,
      pitch_action_range=_PITCH_ROLL_ACTION_RANGE,
      roll_action_range=_PITCH_ROLL_ACTION_RANGE,
      base_velocity_action_range=_BASE_VELOCITY_ACTION_RANGE,
      base_twist_action_range=_BASE_TWIST_SPEED_ACTION_RANGE,
      base_height_action_range=_BASE_HEIGHT_ACTION_RANGE,
      policy_output_swing_action_delta=False,
      swing_target_delta_range=_SWING_TARGET_DELTA_ACTION_RANGE,
      swing_clearance_delta_range=_SWING_CLEARANCE_DELTA_RANGE,
      foot_friction_coeff=0.5,
      contact_detection_force_threshold=0.0,
      locomotion_gait=Gait.WALK,
      target_horizontal_com_velocity_heuristic=BaseTargetHorizontalComVelocityHeuristic(
      ),
      robot_mass_in_mpc=235.0 / 9.8,
      control_frequency=20,
      com_velocity_estimator_class=imu_based_com_velocity_estimator
      .IMUBasedCOMVelocityEstimator):
    """Initializes the wrapper.

    Args:
      gym_env: the wrapped gym environment.
      swing_target_action_range: range for the swing targets specified before
        each swing.
      swing_clearance_action_range: range for the swing clearance.
      pitch_action_range: range for the target body pitch.
      roll_action_range: range for the target body roll.
      base_velocity_action_range: range for the base velocity.
      base_twist_action_range: range for the base twist.
      base_height_action_range: range for the base height.
      policy_output_swing_action_delta: whether to allow the policy to output an
        adjustment to the last swing target during the swing motion.
      swing_target_delta_range: range for the adjustment of the swing target.
      swing_clearance_delta_range: range for the swing clearance adjustments.
      foot_friction_coeff: friction on the feet.
      contact_detection_force_threshold: Threshold of the contact sensor for
        determining whether a foot is in contact. Use 20 for real robot and 0
        for simulation.
      locomotion_gait: Gait to be used.
      target_horizontal_com_velocity_heuristic: .
      robot_mass_in_mpc: mass of the robot used in MPC.
      control_frequency: frequency of querying the policy. The internal MPC
        controller can have higher frequency. Note that the policy outputs a
        swing target and clearance at each query, however, it is only used by
        the environment at the beginning of each swing phase (when
        need_new_swing_target is 1).
      com_velocity_estimator_class: class of the com velocity estimator. Use
        IMUBasedCOMVelocityEstimator for estimating velocity from IMU sensor and
        contact states. Use COMVelocityEstimator for using the ground-truth com
        velocity (e.g. when mocap is available).
    """
    self._gym_env = gym_env
    self._time_per_control_step = 1.0 / control_frequency
    self._foot_friction_coeff = foot_friction_coeff
    self._contact_detection_force_threshold = contact_detection_force_threshold
    self._locomotion_gait = locomotion_gait
    self._policy_output_swing_action_delta = policy_output_swing_action_delta
    self._target_horizontal_com_velocity_heuristic = target_horizontal_com_velocity_heuristic

    self.last_action = None

    self._configure_action_space(
        swing_target_action_range, swing_clearance_action_range,
        pitch_action_range, roll_action_range, base_velocity_action_range,
        base_twist_action_range, base_height_action_range,
        policy_output_swing_action_delta, swing_target_delta_range,
        swing_clearance_delta_range)

    self._configure_observation_space()

    # Set up the MPC controller
    stance_duration, duty_factor, initial_phase = _select_gait(locomotion_gait)
    self._gait_generator = openloop_gait_generator.OpenloopGaitGenerator(
        self._gym_env.robot, stance_duration, duty_factor, initial_phase,
        contact_detection_force_threshold)
    self._com_velocity_estimator = com_velocity_estimator_class(
        self._gym_env.robot)
    self._com_height_estimator = com_height_estimator.COMHeightEstimator(
        self._gym_env.robot)
    self._state_estimator = multi_state_estimator.MultiStateEstimator(
        self._gym_env.robot,
        state_estimators=[
            self._com_velocity_estimator, self._com_height_estimator
        ])
    self._stance_controller = torque_stance_leg_controller.TorqueStanceLegController(
        self._gym_env.robot,
        self._gait_generator,
        self._state_estimator,
        desired_speed=np.array(_DEFAULT_BASE_SPEED),
        desired_twisting_speed=_DEFAULT_BASE_TWIST_SPEED,
        desired_body_height=_DEFAULT_BODY_HEIGHT,
        desired_roll_pitch=np.array(_DEFAULT_ROLL_PITCH),
        body_mass=robot_mass_in_mpc)

  def _configure_observation_space(self):
    """Configure the observation space."""
    self.observation_space.spaces["gait_phases"] = gym.spaces.Box(
        np.array([-1.0] * 4), np.array([1.0] * 4))
    self.observation_space.spaces["feet_states"] = gym.spaces.Box(
        np.array([-2.0] * 4), np.array([2.0] * 4))
    self.observation_space.spaces["need_new_swing_target"] = gym.spaces.Box(
        np.array([0.0] * 4), np.array([1.0] * 4))
    self.observation_space.spaces["estimated_base_speed"] = gym.spaces.Box(
        np.array([-1.0, -1.0, -1.0]), np.array([1.0, 1.0, 1.0]))
    self.observation_space.spaces["estimated_body_height"] = gym.spaces.Box(
        np.array([0.35]), np.array([0.5]))
    self.observation_space.spaces["heuristics_com_velocity"] = gym.spaces.Box(
        np.array([-1.0] * 2), np.array([1.0] * 2))
    self.observation_space.spaces["current_toe_target"] = gym.spaces.Box(
        np.array([-1.0, -1.0, -1.0] * _NUM_LEGS),
        np.array([1.0, 1.0, 1.0] * _NUM_LEGS))

    # Needed so that LastActionSensor uses the correct action space.
    for s in self.all_sensors():
      s.on_reset(self)
    for sensor in self.all_sensors():
      if sensor.get_name() not in self._gym_config.ignored_sensor_list:
        if hasattr(sensor, "observation_space"):
          self.observation_space.spaces[
              sensor.get_name()] = sensor.observation_space

    self.task.reset(self)
    if hasattr(self.task, "observation_space"):
      self.observation_space.spaces[
          self.task.get_name()] = self.task.observation_space

  def _configure_action_space(self, swing_target_action_range,
                              swing_clearance_action_range, pitch_action_range,
                              roll_action_range, base_velocity_action_range,
                              base_twist_action_range, base_height_action_range,
                              policy_output_swing_action_delta,
                              swing_target_delta_range,
                              swing_clearance_delta_range):
    """Configure the action space."""
    ac_lb = np.array([])
    ac_ub = np.array([])

    # Index of different part of the actions within the action array.
    self._action_start_indices = {}
    self._action_dimensions = {}
    self._action_names = []

    # Swing targets and swing clearance.
    for leg_id in range(_NUM_LEGS):
      self._action_start_indices["swing_targets_" + str(leg_id)] = len(ac_lb)
      self._action_dimensions["swing_targets_" + str(leg_id)] = len(
          swing_target_action_range[0])
      self._action_names.append("swing_targets_" + str(leg_id))
      ac_lb = np.concatenate([ac_lb, swing_target_action_range[0]])
      ac_ub = np.concatenate([ac_ub, swing_target_action_range[1]])
    for leg_id in range(_NUM_LEGS):
      self._action_start_indices["swing_clearance_" + str(leg_id)] = len(ac_lb)
      self._action_dimensions["swing_clearance_" + str(leg_id)] = 1
      self._action_names.append("swing_clearance_" + str(leg_id))
      ac_lb = np.concatenate([ac_lb, [swing_clearance_action_range[0]]])
      ac_ub = np.concatenate([ac_ub, [swing_clearance_action_range[1]]])

    # Delta to the swing targets and clearance.
    if policy_output_swing_action_delta:
      for leg_id in range(_NUM_LEGS):
        self._action_start_indices["swing_targets_delta_" +
                                   str(leg_id)] = len(ac_lb)
        self._action_dimensions["swing_targets_delta_" + str(leg_id)] = len(
            swing_target_delta_range[0])
        self._action_names.append("swing_targets_delta_" + str(leg_id))
        ac_lb = np.concatenate([ac_lb, swing_target_delta_range[0]])
        ac_ub = np.concatenate([ac_ub, swing_target_delta_range[1]])
      for leg_id in range(_NUM_LEGS):
        self._action_start_indices["swing_clearance_delta_" +
                                   str(leg_id)] = len(ac_lb)
        self._action_dimensions["swing_clearance_delta_" + str(leg_id)] = 1
        self._action_names.append("swing_clearance_delta_" + str(leg_id))
        ac_lb = np.concatenate([ac_lb, [swing_clearance_delta_range[0]]])
        ac_ub = np.concatenate([ac_ub, [swing_clearance_delta_range[1]]])

    # Desired CoM velocity actions
    # Do not include the action if bounds are all equal to zero
    if not np.all(np.array(base_velocity_action_range) == 0):
      self._action_start_indices["target_base_velocity"] = len(ac_lb)
      self._action_dimensions["target_base_velocity"] = 2
      self._action_names.append("target_base_velocity")
      ac_lb = np.concatenate([
          ac_lb,
          [base_velocity_action_range[0][0], base_velocity_action_range[0][1]]
      ])
      ac_ub = np.concatenate([
          ac_ub,
          [base_velocity_action_range[1][0], base_velocity_action_range[1][1]]
      ])

    # Base twist speed action
    self._action_start_indices["base_twist"] = len(ac_lb)
    self._action_dimensions["base_twist"] = 1
    self._action_names.append("base_twist")
    ac_lb = np.concatenate([ac_lb, [base_twist_action_range[0]]])
    ac_ub = np.concatenate([ac_ub, [base_twist_action_range[1]]])

    # Base height action
    self._action_start_indices["base_height"] = len(ac_lb)
    self._action_dimensions["base_height"] = 1
    self._action_names.append("base_height")
    ac_lb = np.concatenate([ac_lb, [base_height_action_range[0]]])
    ac_ub = np.concatenate([ac_ub, [base_height_action_range[1]]])

    # Roll-pitch action
    self._action_start_indices["base_roll_pitch"] = len(ac_lb)
    self._action_dimensions["base_roll_pitch"] = 2
    self._action_names.append("base_roll_pitch")
    ac_lb = np.concatenate(
        [ac_lb, [roll_action_range[0], pitch_action_range[0]]])
    ac_ub = np.concatenate(
        [ac_ub, [roll_action_range[1], pitch_action_range[1]]])

    self.action_space = gym.spaces.Box(ac_lb, ac_ub)

  def __getattr__(self, attr):
    return getattr(self._gym_env, attr)

  def _fill_observations(self, obs):
    """Fill the additional observations from this wrapper."""
    phase_offset = np.array([
        0 if leg_state == gait_generator_lib.LegState.STANCE else 1
        for leg_state in self._gait_generator.desired_leg_state
    ])
    obs["gait_phases"] = self._gait_generator.normalized_phase - phase_offset
    obs["feet_states"] = []
    for leg_state in self._gait_generator.desired_leg_state:
      if leg_state == gait_generator_lib.LegState.STANCE:
        obs["feet_states"].append(-1)
      if leg_state == gait_generator_lib.LegState.SWING:
        obs["feet_states"].append(1)
      if leg_state == gait_generator_lib.LegState.EARLY_CONTACT:
        obs["feet_states"].append(2)
      if leg_state == gait_generator_lib.LegState.LOSE_CONTACT:
        obs["feet_states"].append(-2)
    obs["need_new_swing_target"] = np.copy(self._need_new_swing_target)
    obs["estimated_base_speed"] = self._state_estimator.com_velocity_body_yaw_aligned_frame
    obs["estimated_body_height"] = [self._state_estimator.estimated_com_height]
    obs["heuristics_com_velocity"] = np.copy(
        self._target_horizontal_com_velocity_heuristic
        .horizontal_com_velocity_heuristic)
    obs["current_toe_target"] = np.copy(self._current_toe_target)

  def _reset_mpc_controller(self):
    """Reset the state of the MPC controller."""
    self._gait_generator.reset(0.0)
    self._state_estimator.reset(0.0)
    self._stance_controller.reset(0.0)
    self._stance_controller.desired_speed = np.array(_DEFAULT_BASE_SPEED)
    self._stance_controller.desired_twisting_speed = _DEFAULT_BASE_TWIST_SPEED
    self._stance_controller.desired_body_height = _DEFAULT_BODY_HEIGHT
    self._stance_controller.desired_roll_pitch = np.array(_DEFAULT_ROLL_PITCH)
    self._mpc_reset_time = self.robot.GetTimeSinceReset()

  def reset(self, **kwargs):
    """Reset the environment."""
    self._gym_env.reset(**kwargs)

    self._reset_mpc_controller()

    self._last_leg_state = copy.copy(self._gait_generator.leg_state)

    self._initial_local_toe_positions = np.array(
        self._gym_env.robot.foot_positions(position_in_world_frame=False))

    self._current_toe_target = np.reshape(
        copy.deepcopy(self._initial_local_toe_positions), 3 * _NUM_LEGS)

    # Record last lift off position for each foot for computing swing
    # trajectory.
    self._lift_off_positions = copy.deepcopy(self._initial_local_toe_positions)

    # Swing command by policy at the beginning of each swing phase.
    self._nominal_swing_leg_commands = []
    for _ in range(_NUM_LEGS):
      self._nominal_swing_leg_commands.append({
          "swing_target": np.array(_DEFAULT_SWING_TARGET),
          "swing_clearance": _DEFAULT_SWING_CLERANCE
      })
    # Actual swing leg commands that incorporated potential delta adjustments
    # from the policy.
    self._actual_swing_leg_commands = []
    for _ in range(_NUM_LEGS):
      self._actual_swing_leg_commands.append({
          "swing_target": np.array(_DEFAULT_SWING_TARGET),
          "swing_clearance": _DEFAULT_SWING_CLERANCE
      })

    self._need_new_swing_target = np.array([0.0] * _NUM_LEGS)

    self._target_horizontal_com_velocity_heuristic.reset()

    self.last_action = []
    for leg_id in range(_NUM_LEGS):
      self.last_action.extend(
          [0.0] * self._action_dimensions["swing_targets_" + str(leg_id)])
    for leg_id in range(_NUM_LEGS):
      self.last_action.extend(
          [0.0] * self._action_dimensions["swing_clearance_" + str(leg_id)])
    if self._policy_output_swing_action_delta:
      for leg_id in range(_NUM_LEGS):
        self.last_action.extend(
            [0.0] *
            self._action_dimensions["swing_targets_delta_" + str(leg_id)])
      for leg_id in range(_NUM_LEGS):
        self.last_action.extend(
            [0.0] *
            self._action_dimensions["swing_clearance_delta_" + str(leg_id)])
    print("=========", len(self.last_action))
    if "target_base_velocity" in self._action_names:
      self.last_action.extend([0.0] *
                              self._action_dimensions["target_base_velocity"])
    self.last_action.extend([0.0] * self._action_dimensions["base_twist"])
    self.last_action.extend([0.0] * self._action_dimensions["base_height"])
    self.last_action.extend([0.0] * self._action_dimensions["base_roll_pitch"])

    # Needed for LastActionSensor to use the correct last_action.
    for s in self.all_sensors():
      s.on_reset(self)
    obs = self._get_observation()
    self._fill_observations(obs)

    self.task.reset(self)
    self._step_counter = 0

    # Change feet friction.
    for link_id in list(
        self.robot.urdf_loader.get_end_effector_id_dict().values()):
      self.pybullet_client.changeDynamics(
          self.robot.robot_id,
          link_id,
          lateralFriction=self._foot_friction_coeff)
    self._observation = obs

    return self._observation

  def _clean_up_action(self, action):
    """Return a cleaned up action to use previous value or zero for components not used.

    Args:
      action: Input action from the policy.

    Returns:
      A cleaned up action where components not used in this step is replaced
        with zero or value from previous steps.
    """
    cleaned_action = np.copy(action)
    for leg_id in range(_NUM_LEGS):
      if not self._need_new_swing_target[leg_id]:
        swing_target_name = "swing_targets_" + str(leg_id)
        swing_targets_start_index = self._action_start_indices[
            swing_target_name]
        swing_targets_end_index = swing_targets_start_index + self._action_dimensions[
            swing_target_name]
        cleaned_action[swing_targets_start_index:
                       swing_targets_end_index] = self.last_action[
                           swing_targets_start_index:swing_targets_end_index]

        swing_clearance_name = "swing_clearance_" + str(leg_id)
        swing_clearance_start_index = self._action_start_indices[
            swing_clearance_name]
        cleaned_action[swing_clearance_start_index] = self.last_action[
            swing_clearance_start_index]
    if self._policy_output_swing_action_delta:
      for leg_id in range(_NUM_LEGS):
        if self._observation["feet_states"][leg_id] != 1:
          swing_target_delta_name = "swing_targets_delta_" + str(leg_id)
          swing_targets_delta_start_index = self._action_start_indices[
              swing_target_delta_name]
          swing_targets_delta_end_index = swing_targets_delta_start_index + self._action_dimensions[
              swing_target_delta_name]
          cleaned_action[
              swing_targets_delta_start_index:
              swing_targets_delta_end_index] = self.last_action[
                  swing_targets_delta_start_index:swing_targets_delta_end_index]

          swing_clearance_delta_name = "swing_clearance_delta_" + str(leg_id)
          swing_clearance_delta_start_index = self._action_start_indices[
              swing_clearance_delta_name]
          cleaned_action[swing_clearance_delta_start_index] = self.last_action[
              swing_clearance_delta_start_index]

    return cleaned_action

  def _get_toe_tracking_target(self, lift_off_position, phase, swing_clearance,
                               landing_position):
    """Get the tracking target for the toes during the swing phase.

    The swing toe will move 70% of the distance in the first half of the swing.
    Intuitely, we want to move the swing foot quickly to the target landing
    location and stay above the ground, in this way the control is more robust
    to perturbations to the body that may cause the swing foot to drop onto
    the ground earlier than expected. This is a common practice similar
    to the MIT cheetah and Marc Raibert's original controllers. After the
    designated swing motion finishes, we also command the foot to go down
    for a short ditance (0.03m). This is to mitigate issues when the swing
    finishes before it touches the ground.

    Args:
      lift_off_position: Local position when the foot leaves ground.
      phase: Normalized phase of the foot in the current swing cycle.
      swing_clearance: Height of the highest point in the swing trajectory.
      landing_position: Target landing position in the local space.

    Returns:
      The interpolated foot target for the current step.
    """
    # Up vector in the world coordinate (without considering yaw).
    rotated_up_vec = np.array(
        self.pybullet_client.multiplyTransforms(
            (0, 0, 0),
            self.pybullet_client.getQuaternionFromEuler(
                (self.robot.base_roll_pitch_yaw[0],
                 self.robot.base_roll_pitch_yaw[1], 0)), (0, 0, 1),
            _UNIT_QUATERNION)[0])

    # Linearly interpolate the trajectory to get the foot target.
    keyframe_timings = [0.0, 0.45, 0.9, 0.9001, 1.0]
    peak_toe_position = 0.3 * lift_off_position + 0.7 * landing_position + rotated_up_vec * swing_clearance
    # TODO(magicmelon): Update the gait generator to warp the gait and keep
    # the swing leg going down until it touches the ground.
    landing_position_pressing_down = landing_position - rotated_up_vec * 0.03
    keyframe_positions = np.array([
        lift_off_position, peak_toe_position, landing_position,
        landing_position_pressing_down, landing_position_pressing_down
    ])

    target_toe_positions = np.array([
        np.interp(phase, keyframe_timings, keyframe_positions[:, 0]),
        np.interp(phase, keyframe_timings, keyframe_positions[:, 1]),
        np.interp(phase, keyframe_timings, keyframe_positions[:, 2])
    ])
    return target_toe_positions

  def step(self, action: Sequence[float]):
    """Steps the wrapped environment.

    Args:
      action:

    Returns:
      The tuple containing the observation, the reward, and the episode
        end indicator.
    """
    self.last_action = self._clean_up_action(action)

    obs, reward, done, info = self._step_motion_controller(self.last_action)
    self._step_counter += 1
    self._fill_observations(obs)
    self._observation = obs

    return obs, reward, done, info

  def _extract_action(self, action, name):
    return action[self.
                  _action_start_indices[name]:self._action_start_indices[name] +
                  self._action_dimensions[name]]

  def _get_swing_foot_ids(self):
    swing_foot_ids = []
    for leg_id in range(_NUM_LEGS):
      if self._gait_generator.leg_state[
          leg_id] == gait_generator_lib.LegState.SWING:
        swing_foot_ids.append(leg_id)
    return swing_foot_ids

  def _update_gait_states_and_flags(self):
    """Update gait-related variables and flags."""
    current_leg_state = self._gait_generator.leg_state
    for leg_id in range(_NUM_LEGS):
      if current_leg_state[
          leg_id] == gait_generator_lib.LegState.SWING and self._last_leg_state[
              leg_id] != gait_generator_lib.LegState.SWING:
        self._lift_off_positions[leg_id] = self._gym_env.robot.foot_positions(
        )[leg_id]
        self._need_new_swing_target[leg_id] = 1
    self._last_leg_state = copy.copy(current_leg_state)

    com_estimate_leg_indices = []
    for leg_id in range(_NUM_LEGS):
      # Use the ones not swinging to estimate the com height
      if leg_id not in self._get_swing_foot_ids():
        com_estimate_leg_indices.append(leg_id)
    self._com_height_estimator.com_estimate_leg_indices = com_estimate_leg_indices
    self._state_estimator.update(self.robot.GetTimeSinceReset() -
                                 self._mpc_reset_time)
    self._gait_generator.update(self.robot.GetTimeSinceReset() -
                                self._mpc_reset_time)

  def _process_action(self, action):
    """Process the action and set relevant variables."""
    # Extract the swing targets from the input action.
    for leg_id in range(_NUM_LEGS):
      if self._need_new_swing_target[leg_id]:
        self._nominal_swing_leg_commands[leg_id]["swing_target"] = (
            self._extract_action(action, "swing_targets_" + str(leg_id)))
        self._nominal_swing_leg_commands[leg_id]["swing_clearance"] = (
            self._extract_action(action, "swing_clearance_" + str(leg_id)))

        self._actual_swing_leg_commands[leg_id]["swing_target"] = np.copy(
            self._nominal_swing_leg_commands[leg_id]["swing_target"])
        self._actual_swing_leg_commands[leg_id][
            "swing_clearance"] = self._nominal_swing_leg_commands[leg_id][
                "swing_clearance"]

    # Reset the flags so the next high level commands are not used until the
    # next swing happens.
    self._need_new_swing_target = np.zeros(_NUM_LEGS)

    # Extract the delta swing targets from the input action.
    if self._policy_output_swing_action_delta:
      for leg_id in self._get_swing_foot_ids():
        self._actual_swing_leg_commands[leg_id]["swing_target"] = (
            self._nominal_swing_leg_commands[leg_id]["swing_target"] +
            self._extract_action(action, "swing_targets_delta_" + str(leg_id)))
        self._actual_swing_leg_commands[leg_id]["swing_clearance"] = (
            self._nominal_swing_leg_commands[leg_id]["swing_clearance"] +
            self._extract_action(action,
                                 "swing_clearance_delta_" + str(leg_id)))
    # Extract the target base movement commands from the input action.
    if "target_base_velocity" in self._action_names:
      self._target_base_velocity_from_policy = self._extract_action(
          action, "target_base_velocity")
    else:
      self._target_base_velocity_from_policy = np.zeros(3)
    desired_twist_speed = self._extract_action(action, "base_twist")
    desired_body_height = self._extract_action(action, "base_height")
    desired_roll_pitch = self._extract_action(action, "base_roll_pitch")

    self._stance_controller.desired_twisting_speed = desired_twist_speed
    self._stance_controller.desired_body_height = desired_body_height
    self._stance_controller.desired_roll_pitch = desired_roll_pitch

  def _compute_swing_action(self):
    """Compute actions for the swing legs."""
    local_toe_positions = np.array(
        self._gym_env.robot.foot_positions(position_in_world_frame=False))
    toe_positions_over_time = copy.deepcopy(local_toe_positions)

    for leg_id in self._get_swing_foot_ids():
      target_toe_positions_local = self._get_toe_tracking_target(
          self._lift_off_positions[leg_id],
          self._gait_generator.normalized_phase[leg_id],
          self._actual_swing_leg_commands[leg_id]["swing_clearance"],
          self._actual_swing_leg_commands[leg_id]["swing_target"] +
          self._initial_local_toe_positions[leg_id])
      toe_positions_over_time[leg_id] = target_toe_positions_local
    self._current_toe_target = np.reshape(
        copy.deepcopy(toe_positions_over_time), 3 * _NUM_LEGS)
    return np.array(
        self.robot.motor_angles_from_foot_positions(
            toe_positions_over_time, position_in_world_frame=False)[1])

  def _compute_stance_action(self):
    """Compute actions for the stance legs."""

    # update target com velocity by combining policy output and heuristics
    hip_relative_swing_targets = []
    swing_durations = []
    com_horizontal_velocity = np.array(
        self._state_estimator.com_velocity_body_yaw_aligned_frame)[0:2]
    for swing_id in self._get_swing_foot_ids():
      hip_relative_swing_targets.append(
          np.array([
              self._actual_swing_leg_commands[swing_id]["swing_target"][0],
              self._actual_swing_leg_commands[swing_id]["swing_target"][1]
          ]))
      swing_durations.append(self._gait_generator.swing_duration[swing_id])

    self._target_horizontal_com_velocity_heuristic.update_horizontal_com_velocity_heuristic(
        hip_relative_swing_targets, com_horizontal_velocity, swing_durations)
    self._stance_controller.desired_speed = np.array(
        self._target_base_velocity_from_policy)
    self._stance_controller.desired_speed[
        0:
        2] += self._target_horizontal_com_velocity_heuristic.horizontal_com_velocity_heuristic

    # compute actions for the stance leg
    return self._stance_controller.get_action()

  def _combine_swing_stance_action(self, swing_action, stance_action):
    """Combine stance and swing leg actions."""
    feet_contact_states = copy.copy(self._gait_generator.leg_state)
    actions = []
    for leg_id in range(_NUM_LEGS):
      if leg_id in self._get_swing_foot_ids(
      ) and feet_contact_states[leg_id] == gait_generator_lib.LegState.SWING:
        for motor_id_in_leg in range(_MOTORS_PER_LEG):
          actions.extend(
              (swing_action[leg_id * _MOTORS_PER_LEG + motor_id_in_leg],
               _MOTOR_KP[leg_id * _MOTORS_PER_LEG + motor_id_in_leg], 0.0,
               _MOTOR_KD[leg_id * _MOTORS_PER_LEG + motor_id_in_leg], 0.0))
      else:
        for motor_id_in_leg in range(_MOTORS_PER_LEG):
          actions.extend(stance_action[leg_id * _MOTORS_PER_LEG +
                                       motor_id_in_leg])
    return actions

  def _step_motion_controller(self, action):
    """Run the MPC controller to advance the robot's state."""

    self._process_action(action)

    # run MPC-based control for a certain amount of time
    total_reward = 0.0
    start_time = self._gym_env.robot.GetTimeSinceReset()
    while self._gym_env.robot.GetTimeSinceReset(
    ) - start_time < self._time_per_control_step:
      self._update_gait_states_and_flags()

      swing_action = self._compute_swing_action()

      stance_action = self._compute_stance_action()

      actions = self._combine_swing_stance_action(swing_action, stance_action)

      obs, rew, done, info = self._gym_env.step(actions)
      if self._stance_controller.qp_solver_fail:
        done = True
      total_reward += rew

      if done:
        return obs, 0.0, done, info

    return obs, total_reward, done, info

  @property
  def observation(self):
    return self._observation

  @property
  def env_step_counter(self):
    return self._step_counter



import os, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(os.path.dirname(currentdir))
os.sys.path.insert(0, parentdir)

import pybullet as p
import numpy as np
import copy
import math
import pybullet_data


class Kuka:

  def __init__(self, urdfRootPath=pybullet_data.getDataPath(), timeStep=0.01):
    self.urdfRootPath = urdfRootPath
    self.timeStep = timeStep
    self.maxVelocity = .35
    self.maxForce = 200.
    self.fingerAForce = 2
    self.fingerBForce = 2.5
    self.fingerTipForce = 2
    self.useInverseKinematics = 1
    self.useSimulation = 1
    self.useNullSpace = 21
    self.useOrientation = 1
    self.kukaEndEffectorIndex = 6
    self.kukaGripperIndex = 7
    #lower limits for null space
    self.ll = [-.967, -2, -2.96, 0.19, -2.96, -2.09, -3.05]
    #upper limits for null space
    self.ul = [.967, 2, 2.96, 2.29, 2.96, 2.09, 3.05]
    #joint ranges for null space
    self.jr = [5.8, 4, 5.8, 4, 5.8, 4, 6]
    #restposes for null space
    self.rp = [0, 0, 0, 0.5 * math.pi, 0, -math.pi * 0.5 * 0.66, 0]
    #joint damping coefficents
    self.jd = [
        0.00001, 0.00001, 0.00001, 0.00001, 0.00001, 0.00001, 0.00001, 0.00001, 0.00001, 0.00001,
        0.00001, 0.00001, 0.00001, 0.00001
    ]
    self.reset()

  def reset(self):
    objects = p.loadSDF(os.path.join(self.urdfRootPath, "kuka_iiwa/kuka_with_gripper2.sdf"))
    self.kukaUid = objects[0]
    #for i in range (p.getNumJoints(self.kukaUid)):
    #  print(p.getJointInfo(self.kukaUid,i))
    p.resetBasePositionAndOrientation(self.kukaUid, [-0.100000, 0.000000, 0.070000],
                                      [0.000000, 0.000000, 0.000000, 1.000000])
    self.jointPositions = [
        0.006418, 0.413184, -0.011401, -1.589317, 0.005379, 1.137684, -0.006539, 0.000048,
        -0.299912, 0.000000, -0.000043, 0.299960, 0.000000, -0.000200
    ]
    self.numJoints = p.getNumJoints(self.kukaUid)
    for jointIndex in range(self.numJoints):
      p.resetJointState(self.kukaUid, jointIndex, self.jointPositions[jointIndex])
      p.setJointMotorControl2(self.kukaUid,
                              jointIndex,
                              p.POSITION_CONTROL,
                              targetPosition=self.jointPositions[jointIndex],
                              force=self.maxForce)

    self.trayUid = p.loadURDF(os.path.join(self.urdfRootPath, "tray/tray.urdf"), 0.640000,
                              0.075000, -0.190000, 0.000000, 0.000000, 1.000000, 0.000000)
    self.endEffectorPos = [0.537, 0.0, 0.5]
    self.endEffectorAngle = 0

    self.motorNames = []
    self.motorIndices = []

    for i in range(self.numJoints):
      jointInfo = p.getJointInfo(self.kukaUid, i)
      qIndex = jointInfo[3]
      if qIndex > -1:
        #print("motorname")
        #print(jointInfo[1])
        self.motorNames.append(str(jointInfo[1]))
        self.motorIndices.append(i)

  def getActionDimension(self):
    if (self.useInverseKinematics):
      return len(self.motorIndices)
    return 6  #position x,y,z and roll/pitch/yaw euler angles of end effector

  def getObservationDimension(self):
    return len(self.getObservation())

  def getObservation(self):
    observation = []
    state = p.getLinkState(self.kukaUid, self.kukaGripperIndex)
    pos = state[0]
    orn = state[1]
    euler = p.getEulerFromQuaternion(orn)

    observation.extend(list(pos))
    observation.extend(list(euler))

    return observation

  def applyAction(self, motorCommands):

    #print ("self.numJoints")
    #print (self.numJoints)
    if (self.useInverseKinematics):

      dx = motorCommands[0]
      dy = motorCommands[1]
      dz = motorCommands[2]
      da = motorCommands[3]
      fingerAngle = motorCommands[4]

      state = p.getLinkState(self.kukaUid, self.kukaEndEffectorIndex)
      actualEndEffectorPos = state[0]
      #print("pos[2] (getLinkState(kukaEndEffectorIndex)")
      #print(actualEndEffectorPos[2])

      self.endEffectorPos[0] = self.endEffectorPos[0] + dx
      if (self.endEffectorPos[0] > 0.65):
        self.endEffectorPos[0] = 0.65
      if (self.endEffectorPos[0] < 0.50):
        self.endEffectorPos[0] = 0.50
      self.endEffectorPos[1] = self.endEffectorPos[1] + dy
      if (self.endEffectorPos[1] < -0.17):
        self.endEffectorPos[1] = -0.17
      if (self.endEffectorPos[1] > 0.22):
        self.endEffectorPos[1] = 0.22

      #print ("self.endEffectorPos[2]")
      #print (self.endEffectorPos[2])
      #print("actualEndEffectorPos[2]")
      #print(actualEndEffectorPos[2])
      #if (dz<0 or actualEndEffectorPos[2]<0.5):
      self.endEffectorPos[2] = self.endEffectorPos[2] + dz

      self.endEffectorAngle = self.endEffectorAngle + da
      pos = self.endEffectorPos
      orn = p.getQuaternionFromEuler([0, -math.pi, 0])  # -math.pi,yaw])
      if (self.useNullSpace == 1):
        if (self.useOrientation == 1):
          jointPoses = p.calculateInverseKinematics(self.kukaUid, self.kukaEndEffectorIndex, pos,
                                                    orn, self.ll, self.ul, self.jr, self.rp)
        else:
          jointPoses = p.calculateInverseKinematics(self.kukaUid,
                                                    self.kukaEndEffectorIndex,
                                                    pos,
                                                    lowerLimits=self.ll,
                                                    upperLimits=self.ul,
                                                    jointRanges=self.jr,
                                                    restPoses=self.rp)
      else:
        if (self.useOrientation == 1):
          jointPoses = p.calculateInverseKinematics(self.kukaUid,
                                                    self.kukaEndEffectorIndex,
                                                    pos,
                                                    orn,
                                                    jointDamping=self.jd)
        else:
          jointPoses = p.calculateInverseKinematics(self.kukaUid, self.kukaEndEffectorIndex, pos)

      #print("jointPoses")
      #print(jointPoses)
      #print("self.kukaEndEffectorIndex")
      #print(self.kukaEndEffectorIndex)
      if (self.useSimulation):
        for i in range(self.kukaEndEffectorIndex + 1):
          #print(i)
          p.setJointMotorControl2(bodyUniqueId=self.kukaUid,
                                  jointIndex=i,
                                  controlMode=p.POSITION_CONTROL,
                                  targetPosition=jointPoses[i],
                                  targetVelocity=0,
                                  force=self.maxForce,
                                  maxVelocity=self.maxVelocity,
                                  positionGain=0.3,
                                  velocityGain=1)
      else:
        #reset the joint state (ignoring all dynamics, not recommended to use during simulation)
        for i in range(self.numJoints):
          p.resetJointState(self.kukaUid, i, jointPoses[i])
      #fingers
      p.setJointMotorControl2(self.kukaUid,
                              7,
                              p.POSITION_CONTROL,
                              targetPosition=self.endEffectorAngle,
                              force=self.maxForce)
      p.setJointMotorControl2(self.kukaUid,
                              8,
                              p.POSITION_CONTROL,
                              targetPosition=-fingerAngle,
                              force=self.fingerAForce)
      p.setJointMotorControl2(self.kukaUid,
                              11,
                              p.POSITION_CONTROL,
                              targetPosition=fingerAngle,
                              force=self.fingerBForce)

      p.setJointMotorControl2(self.kukaUid,
                              10,
                              p.POSITION_CONTROL,
                              targetPosition=0,
                              force=self.fingerTipForce)
      p.setJointMotorControl2(self.kukaUid,
                              13,
                              p.POSITION_CONTROL,
                              targetPosition=0,
                              force=self.fingerTipForce)

    else:
      for action in range(len(motorCommands)):
        motor = self.motorIndices[action]
        p.setJointMotorControl2(self.kukaUid,
                                motor,
                                p.POSITION_CONTROL,
                                targetPosition=motorCommands[action],
                                force=self.maxForce)



import pybullet as p
import time
import pybullet_data


conid = p.connect(p.SHARED_MEMORY)
if (conid < 0):
  p.connect(p.GUI)

p.setAdditionalSearchPath(pybullet_data.getDataPath())
p.setInternalSimFlags(0)
p.resetSimulation()

p.loadURDF("plane.urdf", useMaximalCoordinates=True)
p.loadURDF("tray/traybox.urdf", useMaximalCoordinates=True)

gravXid = p.addUserDebugParameter("gravityX", -10, 10, 0)
gravYid = p.addUserDebugParameter("gravityY", -10, 10, 0)
gravZid = p.addUserDebugParameter("gravityZ", -10, 10, -10)
p.setPhysicsEngineParameter(numSolverIterations=10)
p.setPhysicsEngineParameter(contactBreakingThreshold=0.001)
p.configureDebugVisualizer(p.COV_ENABLE_RENDERING, 0)
for i in range(10):
  for j in range(10):
    for k in range(10):
      ob = p.loadURDF("sphere_1cm.urdf", [0.02 * i, 0.02 * j, 0.2 + 0.02 * k],
                      useMaximalCoordinates=True)

p.configureDebugVisualizer(p.COV_ENABLE_RENDERING, 1)
p.setGravity(0, 0, -10)

p.setRealTimeSimulation(1)
while True:
  gravX = p.readUserDebugParameter(gravXid)
  gravY = p.readUserDebugParameter(gravYid)
  gravZ = p.readUserDebugParameter(gravZid)
  p.setGravity(gravX, gravY, gravZ)
  time.sleep(0.01)



# Lint as: python3
"""Class for loading and managing a scene in pybullet."""

import enum
from typing import Any, Dict, List, Optional, Sequence, Text
import gin
import numpy as np

from pybullet_envs.minitaur.envs_v2.scenes import world_asset_pb2
from pybullet_envs.minitaur.envs_v2 import base_client
from pybullet_utils import bullet_client
from pybullet_envs.minitaur.robots import autonomous_object



# The 2D coordinates of the corners of a polygon. The corners are specified in
# counterclock-wise direction.
Polygon = Sequence[Sequence[float]]


class ObjectType(enum.Enum):
  """Categories of objects that may be found in a scene."""
  OTHER = 0
  GROUND = 1
  OBSTACLE = 2
  GOAL = 3
  DYNAMIC_OBJECT = 4


@gin.configurable
class SceneBase(object):
  """Class for loading and managing a scene."""

  def __init__(
      self,
      data_root: Text = None,
      dynamic_objects: Sequence[autonomous_object.AutonomousObject] = ()):
    """Initializes SceneBase.

    Args:
      data_root: Root directory for finding object models.
      dynamic_objects: Dynamic objects to be added into the scene.
      crowd_builders: Builders of crowds formed by autonomous objects.
    """
    self._pybullet_client = None
    self._data_root = data_root
    temp_dynamic_objects = list(dynamic_objects)
    
    self._dynamic_objects = tuple(temp_dynamic_objects)
    # Dictionaries and world_asset are declared outside init to make sure they
    # are all reset in _reset_scene_tracking().
    self._reset_scene_tracking()

  def _reset_scene_tracking(self):
    """Clears all scene dicts. Used when the simulation is reset."""
    self._type_to_ids_dict = {object_type: [] for object_type in ObjectType}
    self._type_to_objects_dict = {object_type: [] for object_type in ObjectType}
    self._id_to_type_dict = {}
    self._id_to_object_dict = {}
    self._world_asset = None

  def destroy_scene(self):
    """Destroys contents of scene to get ready for another build_scene call."""
    id_to_remove = list(self._id_to_object_dict.keys())
    for object_id in id_to_remove:
      self.remove_object(object_id)
    self._reset_scene_tracking()

  def build_scene(self, pybullet_client: bullet_client.BulletClient):
    """Loads and positions all scene objects in pybullet.

    Override this function in subclass to implement customized scene. The
    overriding function must call base function first.

    Args:
      pybullet_client: A pybullet client in which the scene will be built.
    """
    self._reset_scene_tracking()
    self._pybullet_client = pybullet_client
    self._init_dynamic_objects()

  def reset(self):
    """The soft reset of scene.

    Unlike "build_scene", this is called at each env.reset() before robot
    resetting. Typically we use this API to do some soft resetting like putting
    objects back to its place. Howevever, for special cases such as P2P multimap
    training, we can reload a different mesh scene once a while.
    """
    pass
    
  def _init_dynamic_objects(self):
    """Adds dynamic objects to scene."""
    for an_object in self._dynamic_objects:
      an_object.set_sim_client(self._pybullet_client)
      self.add_object(an_object.sim_object_id, ObjectType.DYNAMIC_OBJECT,
                      an_object)

  @property
  def pybullet_client(self) -> bullet_client.BulletClient:
    if self._pybullet_client is None:
      raise ValueError("pybullet_client is None; did you call build_scene()?")
    return self._pybullet_client

  @property
  def ground_height(self) -> float:
    """Returns ground height of the scene."""
    return 0.0

  @property
  def ground_ids(self) -> List[int]:
    """Returns the pybullet ids of the ground."""
    return self._type_to_ids_dict[ObjectType.GROUND]

  @property
  def obstacle_ids(self) -> List[int]:
    """Returns the pybullet ids of all obstacles in the scene."""
    return self._type_to_ids_dict[ObjectType.OBSTACLE]

  @property
  def goal_ids(self) -> List[int]:
    """Returns the pybullet ids of any goals in the scene."""
    return self._type_to_ids_dict[ObjectType.GOAL]

  @property
  def dynamic_object_ids(self) -> List[int]:
    """Returns the pybullet ids of dynamic objects."""
    return self._type_to_ids_dict[ObjectType.DYNAMIC_OBJECT]

  @property
  def dynamic_objects(self) -> List[autonomous_object.AutonomousObject]:
    """Returns the dynamic objects python object (AutonomousObject)."""
    return self._type_to_objects_dict[ObjectType.DYNAMIC_OBJECT]

  @property
  def world_asset(self) -> world_asset_pb2.WorldAsset:
    """Returns a proto describing the semantics of the scene.

    If the scene keeps a WorldAsset, then mutating this proto will mutate it for
    everyone. If the scene generates a WorldAsset from _type_to_ids_dict, then
    this is not an issue.
    """
    if self._world_asset:
      return self._world_asset
    return self._dict_to_world_asset(self._type_to_ids_dict)

  def add_object(self,
                 object_id: int,
                 class_label: ObjectType,
                 python_object: Optional[Any] = None):
    """Adds an object to be tracked.

    Does not load anything into pybullet.

    Args:
      object_id: objectUniqueId from pybullet.
      class_label: What type to consider the new object.
      python_object: Associated python object for the pybullet object of
        objectUniqueId == object_id. Environment uses the python object to
        control object in pybullet in these cases. One example is python objects
        of class label DYNAMIC_OBJECT: they are associated with python objects
          of type AutonomousObject.
    """
    if python_object is not None:
      if (isinstance(python_object, autonomous_object.AutonomousObject) and
          python_object.sim_object_id != object_id):
        raise ValueError(
            f"Mismatch object ids, object_id = {object_id}, sim_object_id = "
            f"{python_object.sim_object_id}")
      self._type_to_objects_dict[class_label].append(python_object)
    self._type_to_ids_dict[class_label].append(object_id)
    self._id_to_type_dict[object_id] = class_label
    self._id_to_object_dict[object_id] = python_object

  def remove_object(self, object_id: int):
    """Removes an object from tracking and from pybullet.

    Args:
      object_id: objectUniqueID from pybullet.

    Raises:
      KeyError: if object_id does not exist in the record.
    """
    if object_id not in self._id_to_type_dict.keys():
      raise KeyError(
          f"Object with object_id = {object_id} does not exist in the record.")

    self.pybullet_client.removeBody(object_id)
    object_type = self._id_to_type_dict[object_id]
    self._type_to_ids_dict[object_type].remove(object_id)

    object_to_remove = self.id_to_object(object_id)
    if object_to_remove is not None:
      # Removes item by identity comparison and avoid slow down due to objects
      # with complex equality comparison function. list.remove() compares
      # equality instead of identity.
      for i, an_object in enumerate(self._type_to_objects_dict[object_type]):
        if an_object is object_to_remove:
          del self._type_to_objects_dict[object_type][i]
          break
    del self._id_to_type_dict[object_id]
    del self._id_to_object_dict[object_id]

  def id_to_object(self, object_id: int) -> Any:
    """Returns underlying python object from sim object id.

    Args:
      object_id: objectUniqueID from pybullet.

    Returns:
      None is returned if the sim object does not have a corresponding python
      object.
    """
    return self._id_to_object_dict[object_id]

  def _dict_to_world_asset(
      self, type_to_ids_dict: Dict[ObjectType,
                                   List[int]]) -> world_asset_pb2.WorldAsset:
    """Converts a dictionary to a WorldAsset.

    Args:
      type_to_ids_dict: Dictionary that describes the scene. Keys are
        ObjectTypes and values are lists of integers, where each integer is a
        pybullet id for an object of a given type.

    Returns:
      A WorldAsset proto with the types, locations and bounding boxes of all
      objects in the scene.
    """
    world_asset = world_asset_pb2.WorldAsset()
    for object_type in type_to_ids_dict.keys():
      for obj_id in type_to_ids_dict[object_type]:
        bbox = np.array(self.pybullet_client.getAABB(obj_id))
        bbox_center = np.mean(bbox, axis=0)
        bbox_dimensions = bbox[1] - bbox[0]

        obj = world_asset_pb2.Object()
        obj.id = str(obj_id)
        obj.label = str(object_type)
        obj.bounding_box.center.x = bbox_center[0]
        obj.bounding_box.center.y = bbox_center[1]
        obj.bounding_box.center.z = bbox_center[2]
        obj.bounding_box.dimensions.x = bbox_dimensions[0]
        obj.bounding_box.dimensions.y = bbox_dimensions[1]
        obj.bounding_box.dimensions.z = bbox_dimensions[2]
        world_asset.objects.append(obj)
    return world_asset

  def close(self):
    """Closes the scene at the end of life cycle of the environment."""
    pass

  @property
  def vectorized_map(self) -> Sequence[Polygon]:
    """Returns vectorized map containing a list of polygon obstacles."""
    raise NotImplementedError("vectorized_map is not implemented by default.")



import pybullet
import gym, gym.spaces, gym.utils
import numpy as np
import os, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
os.sys.path.insert(0, parentdir)
import pybullet_data


class XmlBasedRobot:
  """
	Base class for mujoco .xml based agents.
	"""

  self_collision = True

  def __init__(self, robot_name, action_dim, obs_dim, self_collision):
    self.parts = None
    self.objects = []
    self.jdict = None
    self.ordered_joints = None
    self.robot_body = None

    high = np.ones([action_dim], dtype=np.float32)
    self.action_space = gym.spaces.Box(-high, high, dtype=np.float32)
    high = np.inf * np.ones([obs_dim], dtype=np.float32)
    self.observation_space = gym.spaces.Box(-high, high, dtype=np.float32)

    #self.model_xml = model_xml
    self.robot_name = robot_name
    self.self_collision = self_collision

  def addToScene(self, bullet_client, bodies):
    self._p = bullet_client

    if self.parts is not None:
      parts = self.parts
    else:
      parts = {}

    if self.jdict is not None:
      joints = self.jdict
    else:
      joints = {}

    if self.ordered_joints is not None:
      ordered_joints = self.ordered_joints
    else:
      ordered_joints = []

    if np.isscalar(bodies):  # streamline the case where bodies is actually just one body
      bodies = [bodies]

    dump = 0
    for i in range(len(bodies)):
      if self._p.getNumJoints(bodies[i]) == 0:
        part_name, robot_name = self._p.getBodyInfo(bodies[i])
        self.robot_name = robot_name.decode("utf8")
        part_name = part_name.decode("utf8")
        parts[part_name] = BodyPart(self._p, part_name, bodies, i, -1)
      for j in range(self._p.getNumJoints(bodies[i])):
        self._p.setJointMotorControl2(bodies[i],
                                      j,
                                      pybullet.POSITION_CONTROL,
                                      positionGain=0.1,
                                      velocityGain=0.1,
                                      force=0)
        jointInfo = self._p.getJointInfo(bodies[i], j)
        joint_name = jointInfo[1]
        part_name = jointInfo[12]

        joint_name = joint_name.decode("utf8")
        part_name = part_name.decode("utf8")

        if dump: print("ROBOT PART '%s'" % part_name)
        if dump:
          print(
              "ROBOT JOINT '%s'" % joint_name
          )  # limits = %+0.2f..%+0.2f effort=%0.3f speed=%0.3f" % ((joint_name,) + j.limits()) )

        parts[part_name] = BodyPart(self._p, part_name, bodies, i, j)

        if part_name == self.robot_name:
          self.robot_body = parts[part_name]

        if i == 0 and j == 0 and self.robot_body is None:  # if nothing else works, we take this as robot_body
          parts[self.robot_name] = BodyPart(self._p, self.robot_name, bodies, 0, -1)
          self.robot_body = parts[self.robot_name]

        if joint_name[:6] == "ignore":
          Joint(self._p, joint_name, bodies, i, j).disable_motor()
          continue

        if joint_name[:8] != "jointfix":
          joints[joint_name] = Joint(self._p, joint_name, bodies, i, j)
          ordered_joints.append(joints[joint_name])

          joints[joint_name].power_coef = 100.0

        # TODO: Maybe we need this
        # joints[joint_name].power_coef, joints[joint_name].max_velocity = joints[joint_name].limits()[2:4]
        # self.ordered_joints.append(joints[joint_name])
        # self.jdict[joint_name] = joints[joint_name]

    return parts, joints, ordered_joints, self.robot_body

  def reset_pose(self, position, orientation):
    self.parts[self.robot_name].reset_pose(position, orientation)


class MJCFBasedRobot(XmlBasedRobot):
  """
	Base class for mujoco .xml based agents.
	"""

  def __init__(self, model_xml, robot_name, action_dim, obs_dim, self_collision=True):
    XmlBasedRobot.__init__(self, robot_name, action_dim, obs_dim, self_collision)
    self.model_xml = model_xml
    self.doneLoading = 0

  def reset(self, bullet_client):

    self._p = bullet_client
    #print("Created bullet_client with id=", self._p._client)
    if (self.doneLoading == 0):
      self.ordered_joints = []
      self.doneLoading = 1
      if self.self_collision:
        self.objects = self._p.loadMJCF(os.path.join(pybullet_data.getDataPath(), "mjcf",
                                                     self.model_xml),
                                        flags=pybullet.URDF_USE_SELF_COLLISION |
                                        pybullet.URDF_USE_SELF_COLLISION_EXCLUDE_ALL_PARENTS |
                                        pybullet.URDF_GOOGLEY_UNDEFINED_COLORS )
        self.parts, self.jdict, self.ordered_joints, self.robot_body = self.addToScene(
            self._p, self.objects)
      else:
        self.objects = self._p.loadMJCF(
            os.path.join(pybullet_data.getDataPath(), "mjcf", self.model_xml, flags = pybullet.URDF_GOOGLEY_UNDEFINED_COLORS))
        self.parts, self.jdict, self.ordered_joints, self.robot_body = self.addToScene(
            self._p, self.objects)
    self.robot_specific_reset(self._p)

    s = self.calc_state(
    )  # optimization: calc_state() can calculate something in self.* for calc_potential() to use

    return s

  def calc_potential(self):
    return 0


class URDFBasedRobot(XmlBasedRobot):
  """
	Base class for URDF .xml based robots.
	"""

  def __init__(self,
               model_urdf,
               robot_name,
               action_dim,
               obs_dim,
               basePosition=[0, 0, 0],
               baseOrientation=[0, 0, 0, 1],
               fixed_base=False,
               self_collision=False):
    XmlBasedRobot.__init__(self, robot_name, action_dim, obs_dim, self_collision)

    self.model_urdf = model_urdf
    self.basePosition = basePosition
    self.baseOrientation = baseOrientation
    self.fixed_base = fixed_base

  def reset(self, bullet_client):
    self._p = bullet_client
    self.ordered_joints = []

    print(os.path.join(os.path.dirname(__file__), "data", self.model_urdf))

    if self.self_collision:
      self.parts, self.jdict, self.ordered_joints, self.robot_body = self.addToScene(
          self._p,
          self._p.loadURDF(os.path.join(pybullet_data.getDataPath(), self.model_urdf),
                           basePosition=self.basePosition,
                           baseOrientation=self.baseOrientation,
                           useFixedBase=self.fixed_base,
                           flags=pybullet.URDF_USE_SELF_COLLISION | pybullet.URDF_GOOGLEY_UNDEFINED_COLORS))
    else:
      self.parts, self.jdict, self.ordered_joints, self.robot_body = self.addToScene(
          self._p,
          self._p.loadURDF(os.path.join(pybullet_data.getDataPath(), self.model_urdf),
                           basePosition=self.basePosition,
                           baseOrientation=self.baseOrientation,
                           useFixedBase=self.fixed_base, flags = pybullet.URDF_GOOGLEY_UNDEFINED_COLORS))

    self.robot_specific_reset(self._p)

    s = self.calc_state(
    )  # optimization: calc_state() can calculate something in self.* for calc_potential() to use
    self.potential = self.calc_potential()

    return s

  def calc_potential(self):
    return 0


class SDFBasedRobot(XmlBasedRobot):
  """
	Base class for SDF robots in a Scene.
	"""

  def __init__(self,
               model_sdf,
               robot_name,
               action_dim,
               obs_dim,
               basePosition=[0, 0, 0],
               baseOrientation=[0, 0, 0, 1],
               fixed_base=False,
               self_collision=False):
    XmlBasedRobot.__init__(self, robot_name, action_dim, obs_dim, self_collision)

    self.model_sdf = model_sdf
    self.fixed_base = fixed_base

  def reset(self, bullet_client):
    self._p = bullet_client

    self.ordered_joints = []

    self.parts, self.jdict, self.ordered_joints, self.robot_body = self.addToScene(
        self._p,  # TODO: Not sure if this works, try it with kuka
        self._p.loadSDF(os.path.join("models_robot", self.model_sdf)))

    self.robot_specific_reset(self._p)

    s = self.calc_state(
    )  # optimization: calc_state() can calculate something in self.* for calc_potential() to use
    self.potential = self.calc_potential()

    return s

  def calc_potential(self):
    return 0


class Pose_Helper:  # dummy class to comply to original interface

  def __init__(self, body_part):
    self.body_part = body_part

  def xyz(self):
    return self.body_part.current_position()

  def rpy(self):
    return pybullet.getEulerFromQuaternion(self.body_part.current_orientation())

  def orientation(self):
    return self.body_part.current_orientation()


class BodyPart:

  def __init__(self, bullet_client, body_name, bodies, bodyIndex, bodyPartIndex):
    self.bodies = bodies
    self._p = bullet_client
    self.bodyIndex = bodyIndex
    self.bodyPartIndex = bodyPartIndex
    self.initialPosition = self.current_position()
    self.initialOrientation = self.current_orientation()
    self.bp_pose = Pose_Helper(self)

  def state_fields_of_pose_of(
      self, body_id,
      link_id=-1):  # a method you will most probably need a lot to get pose and orientation
    if link_id == -1:
      (x, y, z), (a, b, c, d) = self._p.getBasePositionAndOrientation(body_id)
    else:
      (x, y, z), (a, b, c, d), _, _, _, _ = self._p.getLinkState(body_id, link_id)
    return np.array([x, y, z, a, b, c, d])

  def get_position(self):
    return self.current_position()

  def get_pose(self):
    return self.state_fields_of_pose_of(self.bodies[self.bodyIndex], self.bodyPartIndex)

  def speed(self):
    if self.bodyPartIndex == -1:
      (vx, vy, vz), _ = self._p.getBaseVelocity(self.bodies[self.bodyIndex])
    else:
      (x, y, z), (a, b, c, d), _, _, _, _, (vx, vy, vz), (vr, vp, vy) = self._p.getLinkState(
          self.bodies[self.bodyIndex], self.bodyPartIndex, computeLinkVelocity=1)
    return np.array([vx, vy, vz])

  def current_position(self):
    return self.get_pose()[:3]

  def current_orientation(self):
    return self.get_pose()[3:]

  def get_orientation(self):
    return self.current_orientation()

  def reset_position(self, position):
    self._p.resetBasePositionAndOrientation(self.bodies[self.bodyIndex], position,
                                            self.get_orientation())

  def reset_orientation(self, orientation):
    self._p.resetBasePositionAndOrientation(self.bodies[self.bodyIndex], self.get_position(),
                                            orientation)

  def reset_velocity(self, linearVelocity=[0, 0, 0], angularVelocity=[0, 0, 0]):
    self._p.resetBaseVelocity(self.bodies[self.bodyIndex], linearVelocity, angularVelocity)

  def reset_pose(self, position, orientation):
    self._p.resetBasePositionAndOrientation(self.bodies[self.bodyIndex], position, orientation)

  def pose(self):
    return self.bp_pose

  def contact_list(self):
    return self._p.getContactPoints(self.bodies[self.bodyIndex], -1, self.bodyPartIndex, -1)


class Joint:

  def __init__(self, bullet_client, joint_name, bodies, bodyIndex, jointIndex):
    self.bodies = bodies
    self._p = bullet_client
    self.bodyIndex = bodyIndex
    self.jointIndex = jointIndex
    self.joint_name = joint_name

    jointInfo = self._p.getJointInfo(self.bodies[self.bodyIndex], self.jointIndex)
    self.lowerLimit = jointInfo[8]
    self.upperLimit = jointInfo[9]

    self.power_coeff = 0

  def set_state(self, x, vx):
    self._p.resetJointState(self.bodies[self.bodyIndex], self.jointIndex, x, vx)

  def current_position(self):  # just some synonyme method
    return self.get_state()

  def current_relative_position(self):
    pos, vel = self.get_state()
    pos_mid = 0.5 * (self.lowerLimit + self.upperLimit)
    return (2 * (pos - pos_mid) / (self.upperLimit - self.lowerLimit), 0.1 * vel)

  def get_state(self):
    x, vx, _, _ = self._p.getJointState(self.bodies[self.bodyIndex], self.jointIndex)
    return x, vx

  def get_position(self):
    x, _ = self.get_state()
    return x

  def get_orientation(self):
    _, r = self.get_state()
    return r

  def get_velocity(self):
    _, vx = self.get_state()
    return vx

  def set_position(self, position):
    self._p.setJointMotorControl2(self.bodies[self.bodyIndex],
                                  self.jointIndex,
                                  pybullet.POSITION_CONTROL,
                                  targetPosition=position)

  def set_velocity(self, velocity):
    self._p.setJointMotorControl2(self.bodies[self.bodyIndex],
                                  self.jointIndex,
                                  pybullet.VELOCITY_CONTROL,
                                  targetVelocity=velocity)

  def set_motor_torque(self, torque):  # just some synonyme method
    self.set_torque(torque)

  def set_torque(self, torque):
    self._p.setJointMotorControl2(bodyIndex=self.bodies[self.bodyIndex],
                                  jointIndex=self.jointIndex,
                                  controlMode=pybullet.TORQUE_CONTROL,
                                  force=torque)  #, positionGain=0.1, velocityGain=0.1)

  def reset_current_position(self, position, velocity):  # just some synonyme method
    self.reset_position(position, velocity)

  def reset_position(self, position, velocity):
    self._p.resetJointState(self.bodies[self.bodyIndex],
                            self.jointIndex,
                            targetValue=position,
                            targetVelocity=velocity)
    self.disable_motor()

  def disable_motor(self):
    self._p.setJointMotorControl2(self.bodies[self.bodyIndex],
                                  self.jointIndex,
                                  controlMode=pybullet.POSITION_CONTROL,
                                  targetPosition=0,
                                  targetVelocity=0,
                                  positionGain=0.1,
                                  velocityGain=0.1,
                                  force=0)



# Lint as: python3
"""A state machine that steps each foot for a static gait. Experimental code."""

import copy
import math

import numpy as np


class StepInput(object):

  def __init__(self):
    self.base_com_pos = np.array([0, 0, 0])
    self.base_com_orn = np.array([0, 0, 0, 1])
    self.toe_pos_world = np.array([0, 0, 0] * 4)
    self.new_pos_world = np.array([0, 0, 0])


class StepOutput(object):

  def __init__(self, new_toe_pos_world):
    self.new_toe_pos_world = new_toe_pos_world


class FootStepper(object):
  """This class computes desired foot placement for a quadruped robot."""

  def __init__(self, bullet_client, toe_ids, toe_pos_local_ref):
    self.bullet_client = bullet_client
    self.state_time = 0.
    self.toe_ids = toe_ids
    self.toe_pos_local_ref = toe_pos_local_ref
    self.sphere_uid = self.bullet_client.loadURDF(
        "sphere_small.urdf", [0, 0, 0], useFixedBase=True)
    self.is_far = True
    self.max_shift = 0.0008
    self.far_bound = 0.005
    self.close_bound = 0.03

    self.move_swing_foot = False
    self.amp = 0.2
    alpha = 1

    # Loads/draws spheres for debugging purpose. The spheres visualize the
    # target COM, the current COM and the target foothold location.
    self.sphere_uid_centroid = self.bullet_client.loadURDF(
        "sphere_small.urdf", [0, 0, 0], useFixedBase=True)
    self.bullet_client.changeVisualShape(
        self.sphere_uid_centroid, -1, rgbaColor=[1, 1, 0, alpha])

    # Disable collision since visualization spheres should not collide with the
    # robot.
    self.bullet_client.setCollisionFilterGroupMask(self.sphere_uid_centroid, -1,
                                                   0, 0)

    self.sphere_uid_com = self.bullet_client.loadURDF(
        "sphere_small.urdf", [0, 0, 0], useFixedBase=True)
    self.bullet_client.changeVisualShape(
        self.sphere_uid_com, -1, rgbaColor=[1, 0, 1, alpha])
    self.bullet_client.setCollisionFilterGroupMask(self.sphere_uid_com, -1, 0,
                                                   0)

    self.bullet_client.setCollisionFilterGroupMask(self.sphere_uid, -1, 0, 0)
    self.feetindices = [1, 3, 0, 2]
    self.swing_foot_index1 = 0
    self.swing_foot_index = self.feetindices[self.swing_foot_index1]
    self.colors = [[1, 0, 0, 1], [0, 1, 0, 1], [0, 0, 1, 1], [1, 1, 1, 1]]
    self.support_vertices = [[1, 2, 3], [0, 2, 3], [0, 1, 3], [0, 1, 2]]
    self.local_diff_y_threshold = 0.05
    self.local_diff_y = 100
    self.is_far = True
    self.get_reference_pos_swing_foot()

  def next_foot(self):
    self.swing_foot_index1 = (self.swing_foot_index1 + 1) % 4
    self.swing_foot_index = self.feetindices[self.swing_foot_index1]

  def swing_foot(self):
    self.move_swing_foot = True

  def get_reference_pos_swing_foot(self):
    self.new_pos_local = np.array(
        self.toe_pos_local_ref[self.swing_foot_index])
    return self.new_pos_local

  def set_reference_pos_swing_foot(self, new_pos_local):
    self.new_pos_local = new_pos_local

  def is_com_stable(self):
    ld2 = self.local_diff_y * self.local_diff_y
    yaw_ok = ld2 < (self.local_diff_y_threshold * self.local_diff_y_threshold)
    com_ok = not self.is_far
    return com_ok and yaw_ok

  def update(self, step_input):
    """Updates the state machine and toe movements per state."""
    base_com_pos = step_input.base_com_pos
    base_com_orn = step_input.base_com_orn
    base_com_pos_inv, base_com_orn_inv = self.bullet_client.invertTransform(
        base_com_pos, base_com_orn)

    dt = step_input.dt
    self.bullet_client.resetBasePositionAndOrientation(self.sphere_uid,
                                                       step_input.new_pos_world,
                                                       [0, 0, 0, 1])
    self.bullet_client.changeVisualShape(
        self.sphere_uid, -1, rgbaColor=self.colors[self.swing_foot_index])

    all_toes_pos_locals = []
    for toe_pos_world in step_input.toe_pos_world:
      toe_pos_local, _ = self.bullet_client.multiplyTransforms(
          base_com_pos_inv, base_com_orn_inv, toe_pos_world, [0, 0, 0, 1])
      all_toes_pos_locals.append(toe_pos_local)
    all_toes_pos_locals = np.array(all_toes_pos_locals)
    centroid_world = np.zeros(3)
    for v in self.support_vertices[self.swing_foot_index]:
      vtx_pos_world = step_input.toe_pos_world[v]
      centroid_world += vtx_pos_world
    centroid_world /= 3.

    sphere_z_offset = 0.05
    self.diff_world = base_com_pos - centroid_world
    self.diff_world[2] = 0.
    self.bullet_client.resetBasePositionAndOrientation(self.sphere_uid_centroid,
                                                       centroid_world,
                                                       [0, 0, 0, 1])
    self.bullet_client.resetBasePositionAndOrientation(
        self.sphere_uid_com,
        [base_com_pos[0], base_com_pos[1], sphere_z_offset], [0, 0, 0, 1])

    l = np.linalg.norm(self.diff_world)
    if self.is_far:
      bound = self.far_bound
    else:
      bound = self.close_bound

    if l > bound:
      self.diff_world *= self.max_shift * 0.5 / l
      if not self.is_far:
        self.is_far = True
    else:
      if self.is_far:
        self.is_far = False

    if not self.is_far:
      self.diff_world = np.zeros(3)
    for i in range(len(self.toe_pos_local_ref)):
      toe = self.toe_pos_local_ref[i]
      toe = [
          toe[0] + self.diff_world[0], toe[1] + self.diff_world[1],
          toe[2] + self.diff_world[2]
      ]
      self.toe_pos_local_ref[i] = toe

    self.local_diff_y = self.toe_pos_local_ref[0][
        1] + self.toe_pos_local_ref[1][1] - self.toe_pos_local_ref[
            2][1] - self.toe_pos_local_ref[3][1]

    self.yaw = 0
    if self.local_diff_y < -self.local_diff_y_threshold:
      self.yaw = 0.001
    if self.local_diff_y > self.local_diff_y_threshold:
      self.yaw = -0.001

    yaw_trans = self.bullet_client.getQuaternionFromEuler([0, 0, self.yaw])

    if not self.is_far:
      for i in range(len(self.toe_pos_local_ref)):
        toe = self.toe_pos_local_ref[i]
        toe, _ = self.bullet_client.multiplyTransforms([0, 0, 0], yaw_trans,
                                                       toe, [0, 0, 0, 1])
        self.toe_pos_local_ref[i] = toe

    new_toe_pos_world = []

    # Moves the swing foot to the target location.
    if self.move_swing_foot:
      if self.state_time <= 1:
        self.state_time += 4 * dt
    if self.state_time >= 1:
      self.move_swing_foot = False
      self.state_time = 0
      self.toe_pos_local_ref[self.swing_foot_index] = self.new_pos_local
    toe_pos_local_ref_copy = copy.deepcopy(self.toe_pos_local_ref)
    old_pos = self.toe_pos_local_ref[self.swing_foot_index]
    new_pos = [
        old_pos[0] * (1 - self.state_time) + self.new_pos_local[0] *
        (self.state_time), old_pos[1] * (1 - self.state_time) +
        self.new_pos_local[1] * (self.state_time),
        old_pos[2] * (1 - self.state_time) + self.new_pos_local[2] *
        (self.state_time) + self.amp * math.sin(self.state_time * math.pi)
    ]
    toe_pos_local_ref_copy[self.swing_foot_index] = new_pos
    for toe_pos_local in toe_pos_local_ref_copy:
      new_toe_pos_world.append(self.bullet_client.multiplyTransforms(
          base_com_pos, base_com_orn, toe_pos_local, [0, 0, 0, 1])[0])

    step_output = StepOutput(new_toe_pos_world)
    return step_output



"""Tests for pybullet_envs.minitaur.envs.locomotion_gym_config."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import gin
from pybullet_envs.minitaur.envs_v2 import locomotion_gym_config
import unittest
from absl.testing import parameterized



class LocomotionGymConfigTest(unittest.TestCase):

  def testSimulationParametersFromGinString(self):
    config_text = (
        'import pybullet_envs.minitaur'
        '.envs_v2.locomotion_gym_config\n'
        'locomotion_gym_config.SimulationParameters.sim_time_step_s = 0.005\n'
        'locomotion_gym_config.SimulationParameters.camera_distance = 5.0\n'
        'locomotion_gym_config.SimulationParameters.camera_yaw = 10\n'
        'locomotion_gym_config.SimulationParameters.camera_pitch = -50\n'
    )
    gin.parse_config(config_text)

    cfg = locomotion_gym_config.SimulationParameters()
    self.assertEqual(cfg.sim_time_step_s, 0.005)
    self.assertFalse(cfg.enable_hard_reset)
    self.assertEqual(cfg.camera_distance, 5.0)
    self.assertEqual(cfg.camera_yaw, 10)
    self.assertEqual(cfg.camera_pitch, -50)

  def testScalarFieldFromGinString(self):
    config_text = (
        'import pybullet_envs.minitaur'
        '.envs_v2.locomotion_gym_config\n'
        'locomotion_gym_config.ScalarField.name = "MotorUpperLimit"\n'
        'locomotion_gym_config.ScalarField.upper_bound = 1.0\n'
        'locomotion_gym_config.ScalarField.lower_bound = -1.0\n'
    )
    gin.parse_config(config_text)

    cfg = locomotion_gym_config.ScalarField()
    self.assertEqual(cfg.name, 'MotorUpperLimit')
    self.assertEqual(cfg.upper_bound, 1.0)
    self.assertEqual(cfg.lower_bound, -1.0)

  def testLocomotionGymConfigFromGinString(self):
    config_text = (
        'import pybullet_envs.minitaur'
        '.envs_v2.locomotion_gym_config\n'
        # SimulationParameters
        'locomotion_gym_config.SimulationParameters.sim_time_step_s = 0.005\n'
        # Actions
        'Motor/locomotion_gym_config.ScalarField.name = "MotorUpperLimit"\n'
        'Motor/locomotion_gym_config.ScalarField.upper_bound = 1.0\n'
        'Motor/locomotion_gym_config.ScalarField.lower_bound = -1.0\n'
        # LocomotionGymConfigs
        'locomotion_gym_config.LocomotionGymConfig.simulation_parameters = '
        '@locomotion_gym_config.SimulationParameters()\n'
        'locomotion_gym_config.LocomotionGymConfig.actions = ['
        '@Motor/locomotion_gym_config.ScalarField()]\n'
        'locomotion_gym_config.LocomotionGymConfig.ignored_sensor_list = ['
        '"Collisions"]\n')
    gin.parse_config(config_text)

    cfg = locomotion_gym_config.LocomotionGymConfig()
    self.assertEqual(cfg.simulation_parameters.sim_time_step_s, 0.005)
    self.assertEqual(cfg.actions[0].upper_bound, 1.0)
    self.assertEqual(cfg.ignored_sensor_list, ['Collisions'])


if __name__ == '__main__':
  unittest.main()



import pybullet as p
import time
import pybullet_data

p.connect(p.GUI)
p.setAdditionalSearchPath(pybullet_data.getDataPath())
#p.setPhysicsEngineParameter(constraintSolverType=p.CONSTRAINT_SOLVER_LCP_PGS, globalCFM = 0.0001)
p.setPhysicsEngineParameter(constraintSolverType=p.CONSTRAINT_SOLVER_LCP_DANTZIG,
                            globalCFM=0.000001)
#p.setPhysicsEngineParameter(constraintSolverType=p.CONSTRAINT_SOLVER_LCP_PGS, globalCFM = 0.0001)

p.loadURDF("plane.urdf")
radius = 0.025
distance = 1.86
yaw = 135
pitch = -11
targetPos = [0, 0, 0]

p.setPhysicsEngineParameter(solverResidualThreshold=0.001, numSolverIterations=200)
p.resetDebugVisualizerCamera(distance, yaw, pitch, targetPos)
objectId = -1

for i in range(10):
  objectId = p.loadURDF("cube_small.urdf", [1, 1, radius + i * 2 * radius])

p.changeDynamics(objectId, -1, 100)

timeStep = 1. / 240.
p.setGravity(0, 0, -10)
while (p.isConnected()):
  p.stepSimulation()
  time.sleep(timeStep)



# Copyright 2017 The TensorFlow Agents Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Count learnable parameters."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import re

import numpy as np
try:
  import tensorflow.compat.v1 as tf
except Exception:
  import tensorflow as tf


def count_weights(scope=None, exclude=None, graph=None):
  """Count learnable parameters.

  Args:
    scope: Resrict the count to a variable scope.
    exclude: Regex to match variable names to exclude.
    graph: Operate on a graph other than the current default graph.

  Returns:
    Number of learnable parameters as integer.
  """
  if scope:
    scope = scope if scope.endswith('/') else scope + '/'
  graph = graph or tf.get_default_graph()
  vars_ = graph.get_collection(tf.GraphKeys.TRAINABLE_VARIABLES)
  if scope:
    vars_ = [var for var in vars_ if var.name.startswith(scope)]
  if exclude:
    exclude = re.compile(exclude)
    vars_ = [var for var in vars_ if not exclude.match(var.name)]
  shapes = [var.get_shape().as_list() for var in vars_]
  return int(sum(np.prod(shape) for shape in shapes))
